import mythic_container
from mythic_container.logging import logger
import time

MYTHIC_RPC_CALLBACK_SEARCH = "mythic_rpc_callback_search"


class MythicRPCCallbackSearchMessage:
    def __init__(self,
                 AgentCallbackUUID: str = None,
                 AgentCallbackID: int = None,
                 SearchCallbackID: int = None,
                 SearchCallbackDisplayID: int = None,
                 SearchCallbackUUID: str = None,
                 SearchCallbackUser: str = None,
                 SearchCallbackHost: str = None,
                 SearchCallbackPID: int = None,
                 SearchCallbackExtraInfo: str = None,
                 SearchCallbackSleepInfo: str = None,
                 SearchCallbackIP: str = None,
                 SearchCallbackExternalIP: str = None,
                 SearchCallbackIntegrityLevel: int = None,
                 SearchCallbackOs: str = None,
                 SearchCallbackDomain: str = None,
                 SearchCallbackArchitecture: str = None,
                 SearchCallbackDescription: str = None,
                 SearchCallbackPayloadTypes: [str] = None,
                 **kwargs):
        self.AgentCallbackID = AgentCallbackID
        self.AgentCallbackUUID = AgentCallbackUUID
        self.SearchCallbackID = SearchCallbackID
        self.SearchCallbackDisplayID = SearchCallbackDisplayID
        self.SearchCallbackUUID = SearchCallbackUUID
        self.SearchCallbackUser = SearchCallbackUser
        self.SearchCallbackHost = SearchCallbackHost
        self.SearchCallbackPID = SearchCallbackPID
        self.SearchCallbackExtraInfo = SearchCallbackExtraInfo
        self.SearchCallbackSleepInfo = SearchCallbackSleepInfo
        self.SearchCallbackIP = SearchCallbackIP
        self.SearchCallbackExternalIP = SearchCallbackExternalIP
        self.SearchCallbackIntegrityLevel = SearchCallbackIntegrityLevel
        self.SearchCallbackOs = SearchCallbackOs
        self.SearchCallbackDomain = SearchCallbackDomain
        self.SearchCallbackArchitecture = SearchCallbackArchitecture
        self.SearchCallbackDescription = SearchCallbackDescription
        self.SearchCallbackPayloadTypes = SearchCallbackPayloadTypes
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "agent_callback_id": self.AgentCallbackUUID,
            "callback_id": self.AgentCallbackID,
            "search_callback_id": self.SearchCallbackID,
            "search_callback_display_id": self.SearchCallbackDisplayID,
            "search_callback_uuid": self.SearchCallbackUUID,
            "user": self.SearchCallbackUser,
            "host": self.SearchCallbackHost,
            "pid": self.SearchCallbackPID,
            "extra_info": self.SearchCallbackExtraInfo,
            "sleep_info": self.SearchCallbackSleepInfo,
            "ip": self.SearchCallbackIP,
            "external_ip": self.SearchCallbackExternalIP,
            "integrity_level": self.SearchCallbackIntegrityLevel,
            "os": self.SearchCallbackOs,
            "domain": self.SearchCallbackDomain,
            "architecture": self.SearchCallbackArchitecture,
            "description": self.SearchCallbackDescription,
            "payload_types": self.SearchCallbackPayloadTypes
        }


class MythicRPCCallbackSearchMessageResult:
    def __init__(self,
                 id: int = None,
                 display_id: int = None,
                 agent_callback_id: str = None,
                 init_callback: time = None,
                 last_checkin: time = None,
                 user: str = None,
                 host: str = None,
                 pid: int = None,
                 ip: str = None,
                 external_ip: str = None,
                 process_name: str = None,
                 description: str = None,
                 operator_id: int = None,
                 active: bool = None,
                 dead: bool = None,
                 registered_payload_uuid: str = None,
                 payloadtype: str = None,
                 integrity_level: int = None,
                 locked: bool = None,
                 locked_operator_id: int = None,
                 operation_id: int = None,
                 crypto_type: str = None,
                 dec_key: str = None,
                 enc_key: str = None,
                 os: str = None,
                 architecture: str = None,
                 domain: str = None,
                 extra_info: str = None,
                 sleep_info: str = None,
                 timestamp: time = None,
                 **kwargs):
        self.ID = id
        self.DisplayID = display_id
        self.AgentCallbackID = agent_callback_id
        self.InitCallback = init_callback
        self.LastCheckin = last_checkin
        self.User = user
        self.Host = host
        self.PID = pid
        self.Ip = ip
        self.ExternalIp = external_ip
        self.ProcessName = process_name
        self.Description = description
        self.OperatorID = operator_id
        self.Active = active
        self.RegisteredPayloadUUID = registered_payload_uuid
        self.IntegrityLevel = integrity_level
        self.Locked = locked
        self.LockedOperatorID = locked_operator_id
        self.OperationID = operation_id
        self.CryptoType = crypto_type
        self.DecKey = dec_key
        self.EncKey = enc_key
        self.Os = os
        self.Architecture = architecture
        self.Domain = domain
        self.ExtraInfo = extra_info
        self.SleepInfo = sleep_info
        self.Timestamp = timestamp
        self.Dead = dead
        self.PayloadType = payloadtype
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")
    def to_json(self):
        return {
            "id": self.ID,
            "display_id": self.DisplayID,
            "agent_callback_id": self.AgentCallbackID,
            "init_callback": self.InitCallback,
            "last_checkin": self.LastCheckin,
            "user": self.User,
            "host": self.Host,
            "pid": self.PID,
            "ip": self.Ip,
            "external_ip": self.ExternalIp,
            "process_name": self.ProcessName,
            "description": self.Description,
            "operator_id": self.OperatorID,
            "active": self.Active,
            "registered_payload_uuid": self.RegisteredPayloadUUID,
            "integrity_level": self.IntegrityLevel,
            "locked": self.Locked,
            "locked_operator_id": self.LockedOperatorID,
            "operation_id": self.OperationID,
            "crypto_type": self.CryptoType,
            "dec_key": self.DecKey,
            "enc_key": self.EncKey,
            "os": self.Os,
            "architecture": self.Architecture,
            "domain": self.Domain,
            "extra_info": self.ExtraInfo,
            "sleep_info": self.SleepInfo,
            "timestamp": self.Timestamp,
            "dead": self.Dead,
            "payloadtype": self.PayloadType
        }


class MythicRPCCallbackSearchMessageResponse:
    Results: list[MythicRPCCallbackSearchMessageResult]

    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 results: list[dict] = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Results = [MythicRPCCallbackSearchMessageResult(**x) for x in results] if results is not None else []
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCCallbackSearch(
        msg: MythicRPCCallbackSearchMessage) -> MythicRPCCallbackSearchMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_CALLBACK_SEARCH,
                                                                            body=msg.to_json())
    return MythicRPCCallbackSearchMessageResponse(**response)



import json
import base64


class ContainerOnStartMessage:
    def __init__(self,
                 container_name: str = "",
                 operation_id: int = 0,
                 server_name: str = "",
                 apitoken: str = "",
                 **kwargs):
        self.ContainerName = container_name
        self.OperationID = operation_id
        self.ServerName = server_name
        self.APIToken = apitoken

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "operation_id": self.OperationID,
            "server_name": self.ServerName,
            "apitoken": self.APIToken
        }


class ContainerOnStartMessageResponse:
    def __init__(self,
                 ContainerName: str = "",
                 EventLogInfoMessage: str = "",
                 EventLogErrorMessage: str = "",
                 RestartInternalServer: bool = False
                 ):
        self.ContainerName = ContainerName
        self.EventLogInfoMessage = EventLogInfoMessage
        self.EventLogErrorMessage = EventLogErrorMessage
        self.RestartInternalServer = RestartInternalServer

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "stdout": self.EventLogInfoMessage,
            "stderr": self.EventLogErrorMessage
        }


class GetFileMessage:
    """Fetch file from C2 Profile's server folder directory

    Attributes:
        Name (str): Name of the C2 Profile
        Filename (str): Name of the file to read

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 container_name: str,
                 filename: str,
                 **kwargs):
        self.Name = container_name
        self.Filename = filename

    def to_json(self):
        return {
            "container_name": self.Name,
            "filename": self.Filename
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class GetFileMessageResponse:
    """Contents or error from fetching a file

    Attributes:
        Success (bool): Did we successfully read the file or not
        Error (str): Error message if we failed to read the file
        Message (str): Contents of the file

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 Message: bytes = b"",
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Message = Message

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "message": base64.b64encode(self.Message).decode(),
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class ListFileMessage:
    """List the contents of a C2 Profile's server folder directory

    Attributes:
        Name (str): Name of the C2 Profile

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 container_name: str,
                 **kwargs):
        self.Name = container_name

    def to_json(self):
        return {
            "container_name": self.Name,
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class ListFileMessageResponse:
    """List of filenames in the C2 Profile's server folder directory

    Attributes:
        Success (bool): Did the list check succeed or fail
        Error (str): Error message if the list check failed
        Files (list[str]): List of filenames

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 Files: list[str] = [],
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Files = Files

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "files": self.Files,
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class RemoveFileMessage:
    """Request to remove a file from a C2 Profile's server folder path

    Attributes:
        Name (str): Name of the C2 Profile
        Filename (str): Name of the file to remove

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 container_name: str,
                 filename: str,
                 **kwargs):
        self.Name = container_name
        self.Filename = filename

    def to_json(self):
        return {
            "container_name": self.Name,
            "filename": self.Filename
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class RemoveFileMessageResponse:
    """Status of removing the file

    Attributes:
        Success (bool): Did the file removal succeed or fail
        Error (str): Error message if the file removal failed

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 **kwargs):
        self.Success = Success
        self.Error = Error

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class WriteFileMessage:
    """Request to write a file to the C2 Profile's server folder directory

    Attributes:
        Name (str): Name of the C2 Profile
        Filename (str): Name of the file to create/overwrite
        Contents (bytes): Contents of the file to write

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 container_name: str,
                 filename: str,
                 contents: str,
                 **kwargs):
        self.Name = container_name
        self.Filename = filename
        self.Contents = base64.b64decode(contents)

    def to_json(self):
        return {
            "container_name": self.Name,
            "filename": self.Filename,
            "contents": base64.b64encode(self.Contents)
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class WriteFileMessageResponse:
    """Status of writing the file to disk

    Attributes:
        Success (bool): Did the file get written or not
        Error (str): Error message if the file failed to write to disk
        Message (str): Informative message about the file getting written

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 Message: bytes = b"",
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Message = Message

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "message": self.Message,
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pushC2GRPC.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10pushC2GRPC.proto\x12\x0epushC2Services\"\xab\x01\n\x16PushC2MessageFromAgent\x12\x15\n\rC2ProfileName\x18\x01 \x01(\t\x12\x10\n\x08RemoteIP\x18\x02 \x01(\t\x12\x0f\n\x07Message\x18\x03 \x01(\x0c\x12\x11\n\tOuterUUID\x18\x04 \x01(\t\x12\x15\n\rBase64Message\x18\x05 \x01(\x0c\x12\x12\n\nTrackingID\x18\x06 \x01(\t\x12\x19\n\x11\x41gentDisconnected\x18\x07 \x01(\x08\"^\n\x17PushC2MessageFromMythic\x12\x0f\n\x07Success\x18\x01 \x01(\x08\x12\r\n\x05\x45rror\x18\x02 \x01(\t\x12\x0f\n\x07Message\x18\x03 \x01(\x0c\x12\x12\n\nTrackingID\x18\x04 \x01(\t2\xef\x01\n\x06PushC2\x12m\n\x14StartPushC2Streaming\x12&.pushC2Services.PushC2MessageFromAgent\x1a\'.pushC2Services.PushC2MessageFromMythic\"\x00(\x01\x30\x01\x12v\n\x1dStartPushC2StreamingOneToMany\x12&.pushC2Services.PushC2MessageFromAgent\x1a\'.pushC2Services.PushC2MessageFromMythic\"\x00(\x01\x30\x01\x42/Z-github.com/its-a-feature/Mythic/grpc/servicesb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'pushC2GRPC_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z-github.com/its-a-feature/Mythic/grpc/services'
  _PUSHC2MESSAGEFROMAGENT._serialized_start=37
  _PUSHC2MESSAGEFROMAGENT._serialized_end=208
  _PUSHC2MESSAGEFROMMYTHIC._serialized_start=210
  _PUSHC2MESSAGEFROMMYTHIC._serialized_end=304
  _PUSHC2._serialized_start=307
  _PUSHC2._serialized_end=546
# @@protoc_insertion_point(module_scope)



import ujson
from . import EventingBase
import traceback
import mythic_container


async def ConditionalEventingCheck(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in EventingBase.eventingServices.items():
            if pt.name == msgDict["container_name"]:
                for conditionalDef in pt.conditional_checks:
                    if conditionalDef.Name == msgDict["function_name"]:
                        try:
                            response = await conditionalDef.Function(EventingBase.ConditionalCheckEventingMessage(**msgDict))
                            response.EventStepInstanceID = msgDict["eventstepinstance_id"]
                        except Exception as e:
                            response = EventingBase.ConditionalCheckEventingMessageResponse(
                                EventStepInstanceID=msgDict["eventstepinstance_id"],
                                StdErr=f"Hit exception trying to call conditional check function: {traceback.format_exc()}\n{e}"
                            )
                        await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                            queue=mythic_container.EVENTING_CONDITIONAL_CHECK_RESPONSE,
                            body=response.to_json()
                        )
                return
    except Exception as e:
        await mythic_container.RabbitmqConnection.SendDictDirectMessage(
            queue=mythic_container.EVENTING_CONDITIONAL_CHECK_RESPONSE,
            body=EventingBase.ConditionalCheckEventingMessageResponse(
                StdErr=f"Hit exception trying to call conditional check function: {traceback.format_exc()}\n{e}"
            ).to_json()
        )


async def CustomFunction(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in EventingBase.eventingServices.items():
            if pt.name == msgDict["container_name"]:
                for conditionalDef in pt.custom_functions:
                    if conditionalDef.Name == msgDict["function_name"]:
                        try:
                            response = await conditionalDef.Function(EventingBase.NewCustomEventingMessage(**msgDict))
                            response.EventStepInstanceID = msgDict["eventstepinstance_id"]
                        except Exception as e:
                            response = EventingBase.NewCustomEventingMessageResponse(
                                EventStepInstanceID=msgDict["eventstepinstance_id"],
                                StdErr=f"Hit exception trying to call custom function function: {traceback.format_exc()}\n{e}",
                            )
                        await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                            queue=mythic_container.EVENTING_CUSTOM_FUNCTION_RESPONSE,
                            body=response.to_json()
                        )
                return
    except Exception as e:
        await mythic_container.RabbitmqConnection.SendDictDirectMessage(
            queue=mythic_container.EVENTING_CUSTOM_FUNCTION_RESPONSE,
            body=EventingBase.NewCustomEventingMessageResponse(
                StdErr=f"Hit exception trying to call custom function function: {traceback.format_exc()}\n{e}"
            ).to_json()
        )


async def TaskIntercept(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in EventingBase.eventingServices.items():
            if pt.name == msgDict["container_name"]:
                if pt.task_intercept_function is not None:
                    try:
                        response = await pt.task_intercept_function(EventingBase.TaskInterceptMessage(**msgDict))
                        response.EventStepInstanceID = msgDict["eventstepinstance_id"]
                    except Exception as e:
                        response = EventingBase.TaskInterceptMessageResponse(
                            EventStepInstanceID=msgDict["eventstepinstance_id"],
                            BlockTask=True,
                            Success=False,
                            StdErr=f"Hit exception trying to call task intercept function: {traceback.format_exc()}\n{e}"
                        )
                    await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                        queue=mythic_container.EVENTING_TASK_INTERCEPT_RESPONSE,
                        body=response.to_json()
                    )
                return
    except Exception as e:
        await mythic_container.RabbitmqConnection.SendDictDirectMessage(
            queue=mythic_container.EVENTING_TASK_INTERCEPT_RESPONSE,
            body=EventingBase.TaskInterceptMessageResponse(
                StdErr=f"Hit exception trying to call task intercept function: {traceback.format_exc()}\n{e}"
            ).to_json()
        )


async def ResponseIntercept(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in EventingBase.eventingServices.items():
            if pt.name == msgDict["container_name"]:
                if pt.response_intercept_function is not None:
                    try:
                        response = await pt.response_intercept_function(EventingBase.ResponseInterceptMessage(**msgDict))
                        response.EventStepInstanceID = msgDict["eventstepinstance_id"]
                    except Exception as e:
                        response = EventingBase.ResponseInterceptMessageResponse(
                            Success=False,
                            StdErr=f"Hit exception trying to call get_idp_metadata function: {traceback.format_exc()}\n{e}"
                        )
                    await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                        queue=mythic_container.EVENTING_RESPONSE_INTERCEPT_RESPONSE,
                        body=response.to_json()
                    )
                return
    except Exception as e:
        await mythic_container.RabbitmqConnection.SendDictDirectMessage(
            queue=mythic_container.EVENTING_RESPONSE_INTERCEPT_RESPONSE,
            body=EventingBase.ResponseInterceptMessageResponse(
                StdErr=f"Hit exception trying to call get_idp_metadata function: {traceback.format_exc()}\n{e}"
            ).to_json()
        )



from collections.abc import Callable, Awaitable
from typing import Union

import mythic_container
from .logging import logger
import base64
import json
from .MythicCommandBase import PTTaskMessageTaskData
from .SharedClasses import ContainerOnStartMessage, ContainerOnStartMessageResponse


class ConditionalCheckEventingMessage:
    def __init__(self,
                 eventstepinstance_id: int = 0,
                 function_name: str = "",
                 container_name: str = "",
                 environment: dict = {},
                 inputs: dict = {},
                 action_data: dict = {},
                 **kwargs):
        self.EventStepInstanceID = eventstepinstance_id
        self.FunctionName = function_name
        self.ContainerName = container_name
        self.Environment = environment
        self.Inputs = inputs
        self.ActionData = action_data

    def to_json(self):
        return {
            "eventstepinstance_id": self.EventStepInstanceID,
            "function_name": self.FunctionName,
            "container_name": self.ContainerName,
            "environment": self.Environment,
            "inputs": self.Inputs,
            "action_data": self.ActionData
        }


class ConditionalCheckEventingMessageResponse:
    def __init__(self,
                 EventStepInstanceID: int = 0,
                 Success: bool = True,
                 StdErr: str = "",
                 StdOut: str = "",
                 Outputs: dict = {},
                 SkipStep: bool = False,
                 **kwargs):
        self.EventStepInstanceID = EventStepInstanceID
        self.Success = Success
        self.StdErr = StdErr
        self.StdOut = StdOut
        self.Outputs = Outputs
        self.SkipStep = SkipStep

    def to_json(self):
        return {
            "eventstepinstance_id": self.EventStepInstanceID,
            "success": self.Success,
            "stderr": self.StdErr,
            "stdout": self.StdOut,
            "outputs": self.Outputs,
            "skip_step": self.SkipStep,
        }


class NewCustomEventingMessage:
    def __init__(self,
                 container_name: str = "",
                 eventstepinstance_id: int = 0,
                 function_name: str = "",
                 environment: dict = {},
                 inputs: dict = {},
                 action_data: dict = {}):
        self.ContainerName = container_name
        self.EventStepInstanceID = eventstepinstance_id
        self.FunctionName = function_name
        self.Environment = environment
        self.Inputs = inputs
        self.ActionData = action_data

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "eventstepinstance_id": self.EventStepInstanceID,
            "function_name": self.FunctionName,
            "environment": self.Environment,
            "inputs": self.Inputs,
            "action_data": self.ActionData
        }


class NewCustomEventingMessageResponse:
    def __init__(self,
                 EventStepInstanceID: int = 0,
                 Success: bool = True,
                 StdErr: str = "",
                 StdOut: str = "",
                 Outputs: dict = {},
                 **kwargs):
        self.EventStepInstanceID = EventStepInstanceID
        self.Success = Success
        self.StdErr = StdErr
        self.StdOut = StdOut
        self.Outputs = Outputs

    def to_json(self):
        return {
            "eventstepinstance_id": self.EventStepInstanceID,
            "success": self.Success,
            "stderr": self.StdErr,
            "stdout": self.StdOut,
            "outputs": self.Outputs,
        }


class ResponseInterceptMessage:
    def __init__(self,
                 eventstepinstance_id: int = 0,
                 response_id: int = 0,
                 callback_id: int = 0,
                 callback_display_id: int = 0,
                 agent_callback_id: str = "",
                 container_name: str = "",
                 environment: dict = {},
                 inputs: dict = {},
                 action_data: dict = {},
                 **kwargs):
        self.ContainerName = container_name
        self.EventStepInstanceID = eventstepinstance_id
        self.ResponseID = response_id
        self.CallbackID = callback_id
        self.CallbackDisplayID = callback_display_id
        self.AgentCallbackID = agent_callback_id
        self.Environment = environment
        self.Inputs = inputs
        self.ActionData = action_data

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "eventstepinstance_id": self.EventStepInstanceID,
            "response_id": self.ResponseID,
            "callback_id": self.CallbackID,
            "callback_display_id": self.CallbackDisplayID,
            "agent_callback_id": self.AgentCallbackID,
            "environment": self.Environment,
            "inputs": self.Inputs,
            "action_data": self.ActionData,
        }


class ResponseInterceptMessageResponse:
    def __init__(self,
                 EventStepInstanceID: int = 0,
                 ResponseID: int = 0,
                 Success: bool = True,
                 StdErr: str = "",
                 StdOut: str = "",
                 Outputs: dict = {},
                 Response: str = "",
                 **kwargs):
        self.EventStepInstanceID = EventStepInstanceID
        self.Success = Success
        self.StdErr = StdErr
        self.StdOut = StdOut
        self.Outputs = Outputs
        self.ResponseID = ResponseID
        self.Response = Response

    def to_json(self):
        return {
            "eventstepinstance_id": self.EventStepInstanceID,
            "success": self.Success,
            "stderr": self.StdErr,
            "stdout": self.StdOut,
            "outputs": self.Outputs,
            "response_id": self.ResponseID,
            "response": self.Response,
        }


class TaskInterceptMessage:
    def __init__(self,
                 eventstepinstance_id: int = 0,
                 task_id: int = 0,
                 callback_id: int = 0,
                 container_name: str = "",
                 environment: dict = {},
                 inputs: dict = {},
                 action_data: dict = {},
                 **kwargs):
        self.ContainerName = container_name
        self.TaskID = task_id
        self.EventStepInstanceID = eventstepinstance_id
        self.CallbackID = callback_id
        self.Environment = environment
        self.Inputs = inputs
        self.ActionData = action_data

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "eventstepinstance_id": self.EventStepInstanceID,
            "task_id": self.TaskID,
            "callback_id": self.CallbackID,
            "environment": self.Environment,
            "inputs": self.Inputs,
            "action_data": self.ActionData,
        }


class TaskInterceptMessageResponse:
    def __init__(self,
                 EventStepInstanceID: int = 0,
                 TaskID: int = 0,
                 Success: bool = True,
                 StdErr: str = "",
                 StdOut: str = "",
                 Outputs: dict = {},
                 BlockTask: bool = False,
                 BypassRole: str = "",
                 BypassMessage: str = "",
                 **kwargs):
        self.EventStepInstanceID = EventStepInstanceID
        self.Success = Success
        self.StdErr = StdErr
        self.StdOut = StdOut
        self.Outputs = Outputs
        self.TaskID = TaskID
        self.BlockTask = BlockTask
        self.BypassRole = BypassRole
        self.BypassMessage = BypassMessage

    def to_json(self):
        return {
            "eventstepinstance_id": self.EventStepInstanceID,
            "success": self.Success,
            "stderr": self.StdErr,
            "stdout": self.StdOut,
            "outputs": self.Outputs,
            "task_id": self.TaskID,
            "block_task": self.BlockTask,
            "bypass_role": self.BypassRole,
            "bypass_message": self.BypassMessage,
        }


class CustomFunctionDefinition:
    def __init__(self,
                 Name: str = "",
                 Description: str = "",
                 Function: Callable[[NewCustomEventingMessage], Awaitable[NewCustomEventingMessageResponse]] = None):
        self.Name = Name
        self.Description = Description
        self.Function = Function

    def to_json(self):
        return {
            "name": self.Name,
            "description": self.Description,
        }


class ConditionalCheckDefinition:
    def __init__(self,
                 Name: str = "",
                 Description: str = "",
                 Function: Callable[[ConditionalCheckEventingMessage], Awaitable[ConditionalCheckEventingMessageResponse]] = None):
        self.Name = Name
        self.Description = Description
        self.Function = Function

    def to_json(self):
        return {
            "name": self.Name,
            "description": self.Description,
        }


class Eventing:
    """Eventing definition class for performing custom eventing actions within Mythic


    Attributes:
        name:
            name for this auth service that appears in the Mythic UI
        description:
            description about what this auth service provides that appears in the Mythic UI
        custom_functions:
            an array of CustomFunctionDefinition entries that provide name, description, and function implementations
        conditional_checks:
            an array of ConditionalCheckDefinition entries that provide name, description, and function implementations
    Functions:
        task_intercept_function:
        response_intercept_function:
    """
    name: str = ""
    description: str = ""

    task_intercept_function: Callable[[TaskInterceptMessage], Awaitable[TaskInterceptMessageResponse]] = None
    response_intercept_function: Callable[
        [ResponseInterceptMessage], Awaitable[ResponseInterceptMessageResponse]] = None
    custom_functions: [CustomFunctionDefinition] = []
    conditional_checks: [ConditionalCheckDefinition] = []

    async def on_container_start(self, message: ContainerOnStartMessage) -> ContainerOnStartMessageResponse:
        return ContainerOnStartMessageResponse(ContainerName=self.name)

    def get_sync_message(self):
        custom = [x.to_json() for x in self.custom_functions]
        conditional = [x.to_json() for x in self.conditional_checks]
        subscriptions = custom + conditional
        if self.task_intercept_function is not None:
            subscriptions.append({
                "name":        "task_intercept",
                "description": "Intercept Task execution before it gets to an agent and potentially block it",
            })
        if self.response_intercept_function is not None:
            subscriptions.append({
                "name":        "response_intercept",
                "description": "Intercept User Output Responses they get sent to the user",
            })
        subscriptions = [json.dumps(x) for x in subscriptions]
        return {
            "name": self.name,
            "type": "eventing",
            "description": self.description,
            "subscriptions": subscriptions
        }


eventingServices: dict[str, Eventing] = {}


async def SendMythicRPCSyncEventing(eventing_name: str) -> bool:
    try:
        eventing_services = Eventing.__subclasses__()
        for cls in eventing_services:
            event = cls()
            if event.name == "":
                continue
            if event.name == eventing_name:
                eventingServices.pop(eventing_name, None)
                eventingServices[event.name] = event
                await mythic_container.mythic_service.syncEventingData(event)
                return True
        return False
    except Exception as e:
        return False


from collections.abc import Callable, Awaitable
from typing import Union

import mythic_container
from .logging import logger
import aiohttp
from .config import settings
import json
from .SharedClasses import ContainerOnStartMessage, ContainerOnStartMessageResponse


class NewCallbackWebhookData:
    """The base information about a new callback within Mythic

    Attributes:
        User (str): The user for the callback
        Host (str): The hostname for the callback (in all caps)
        IPs (list[str]): An array of IP addresses for the callback
        Domain (str): The domain for the callback
        ExternalIP (str): The external IP address for the callback (if known)
        ProcessName (str): The name of the process executing the callback
        PID (int): The Process Identifier for the callback
        OS (str): The operating system information reported back by the callback (not the same as the os you selected when building the payload)
        Architecture (str): The architecture of the process running the callback
        PayloadType (str): The name of the Payload Type for this callback
        Description (str): The description for the callback (defaults to the description for the associtated payload)
        ExtraInfo (str): Freeform additional text that can be set on the callback
        SleepInfo (str): Sleep information that can be stored as part of the callback (not implicitly set)
        DisplayID (int): The ID that the user sees when looking at the callback in Mythic's UI
        ID (int): The unique ID associated with this callback that can be used with RPC calls
        IntegrityLevel (int): The integrity level of this callback (mirrors Windows integrity levels with 0-5 range and 3+ is High integrity)

    """

    def __init__(self,
                 user: str = None,
                 host: str = None,
                 ips: str = None,
                 domain: str = None,
                 external_ip: str = None,
                 process_name: str = None,
                 pid: int = 0,
                 os: str = None,
                 architecture: str = None,
                 agent_type: str = None,
                 description: str = None,
                 extra_info: str = None,
                 sleep_info: str = None,
                 display_id: int = 0,
                 id: int = 0,
                 integrity_level: int = 2):
        self.User = user
        self.Host = host
        self.IPs = json.loads(ips) if ips is not None else None
        self.Domain = domain
        self.ExternalIP = external_ip
        self.ProcessName = process_name
        self.PID = pid
        self.OS = os
        self.Architecture = architecture
        self.PayloadType = agent_type
        self.Description = description
        self.ExtraInfo = extra_info
        self.SleepInfo = sleep_info
        self.DisplayID = display_id
        self.ID = id
        self.IntegrityLevel = integrity_level

    def to_json(self):
        return {
            "user": self.User,
            "host": self.Host,
            "ips": self.IPs,
            "domain": self.Domain,
            "external_ip": self.ExternalIP,
            "process_name": self.ProcessName,
            "pid": self.PID,
            "os": self.OS,
            "architecture": self.Architecture,
            "description": self.Description,
            "extra_info": self.ExtraInfo,
            "sleep_info": self.SleepInfo,
            "display_id": self.DisplayID,
            "id": self.ID,
            "agent_type": self.PayloadType,
            "integrity_level": self.IntegrityLevel
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class NewFeedbackWebhookData:
    """The base definition for new feedback a user is providing about Mythic

    Attributes:
        TaskID (int): The ID of the task associated with this feedback (if provided) that can be used with RPC Calls
        Message (str): The feedback message
        FeedbackType (str): The type of feedback
        DisplayID (int): The display ID of the task that the user would see

    """

    def __init__(self,
                 task_id: int = None,
                 display_id: int = None,
                 message: str = None,
                 feedback_type: str = None,
                 **kwargs):
        self.TaskID = task_id
        self.Message = message
        self.FeedbackType = feedback_type
        self.DisplayID = display_id
        for k, v in kwargs.items():
            logger.info(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "message": self.Message,
            "feedback_type": self.FeedbackType,
            "display_id": self.DisplayID
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class NewStartupWebhookData:
    """The base definition for a notification that Mythic started

    Attributes:
        StartupMessage (str): The message that Mythic started

    """

    def __init__(self,
                 startup_message: str = None,
                 **kwargs):
        self.StartupMessage = startup_message
        for k, v in kwargs.items():
            logger.info(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "startup_message": self.StartupMessage
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class NewAlertWebhookData:
    """The base definition for a notification that there was an alert generated

    Attributes:
        OperatorID (int): The unique ID of the operator that caused this message (if not Mythic)
        Message (str): The alert message
        Source (str): The source of the alert message
        Count (int): The number of times this entry has been seen
        Timestamp (str): When this alert was generated

    """

    def __init__(self,
                 operator_id: int = None,
                 message: str = None,
                 source: str = None,
                 count: int = None,
                 timestamp: str = None,
                 **kwargs):
        self.OperatorID = operator_id
        self.Message = message
        self.Source = source
        self.Count = count
        self.Timestamp = timestamp
        for k, v in kwargs.items():
            logger.info(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "operator_id": self.OperatorID,
            "message": self.Message,
            "source": self.Source,
            "count": self.Count,
            "timestamp": self.Timestamp
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class WebhookMessage:
    """The base definition for webhook message from Mythic

    Attributes:
        OperationID (int): The operation ID associated with this event
        OperationName (str): The name of the operation associated with this event
        OperationWebhook (str): If the operation has a configured webhook url, it's here
        OperationChannel (str): If the operation has a configured webhook channel, it's here
        OperatorUsername (str): The username of the operator that caused this webhook event
        Action (str): The kind of webhook message
        Data: The action-specific data
    """
    Data: Union[dict |
                NewCallbackWebhookData |
                NewFeedbackWebhookData |
                NewStartupWebhookData |
                NewAlertWebhookData |
                None]

    def __init__(self,
                 operation_id: int = None,
                 operation_name: str = None,
                 operation_webhook: str = None,
                 operation_channel: str = None,
                 operator_username: str = None,
                 action: str = None,
                 data: dict = None,
                 server_name: str = None,
                 **kwargs):
        self.OperationID = operation_id
        self.OperationName = operation_name
        self.OperationWebhook = operation_webhook
        self.OperationChannel = operation_channel
        self.OperatorUsername = operator_username
        self.ServerName = server_name
        self.Action = action
        if self.Action == mythic_container.WEBHOOK_TYPE_NEW_CALLBACK:
            self.Data = NewCallbackWebhookData(**data)
        elif self.Action == mythic_container.WEBHOOK_TYPE_NEW_FEEDBACK:
            self.Data = NewFeedbackWebhookData(**data)
        elif self.Action == mythic_container.WEBHOOK_TYPE_NEW_STARTUP:
            self.Data = NewStartupWebhookData(**data)
        elif self.Action == mythic_container.WEBHOOK_TYPE_NEW_ALERT:
            self.Data = NewAlertWebhookData(**data)
        elif self.Action == mythic_container.WEBHOOK_TYPE_NEW_CUSTOM:
            self.Data = data
        else:
            self.Data = data
        for k, v in kwargs.items():
            logger.info(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "operation_id": self.OperationID,
            "operation_name": self.OperationName,
            "operation_webhook": self.OperationWebhook,
            "operation_channel": self.OperationChannel,
            "operator_username": self.OperatorUsername,
            "action": self.Action,
            "server_name": self.ServerName,
            "data": self.Data if isinstance(self.Data, dict) or self.Data is None else self.Data.to_json()
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class Webhook:
    """The base definition for a webhook service to accept webhook messages from Mythic

    In your functions, you can use the sendWebhookMessage function to send the webhook message.

    Attributes:
        webhook_url (str): Custom webhook url to use in case one isn't configured in Mythic
        webhook_channel (str): Custom webhook channel to use in case one isn't configured in Mythic

    Functions:
        new_callback:
            If you want to accept messages about new callbacks, implement this function
        new_feedback:
            If you want to accept messages about new feedback, implement this function
        new_startup:
            If you want to accept messages about Mythic starting, implement this function
        new_alert:
            If you want to accept messages about alert messages from Mythic's operation event log
        new_custom:
            If you want to accept messages that are custom made by scripting
        getWebhookURL:
            This will automatically determine the url to use by looking at the operation configuration, this class' configuration, and any env configurations
        getWebhookChannel:
            This will automatically determine the channel to use by look at the operation configuration, this class' configuartion, and any env configurations
    """
    name: str = ""
    description: str = ""

    webhook_url: str = ""
    webhook_channel: str = ""

    new_callback: Callable[[WebhookMessage], Awaitable[None]] = None
    new_feedback: Callable[[WebhookMessage], Awaitable[None]] = None
    new_startup: Callable[[WebhookMessage], Awaitable[None]] = None
    new_alert: Callable[[WebhookMessage], Awaitable[None]] = None
    new_custom: Callable[[WebhookMessage], Awaitable[None]] = None

    def getWebhookURL(self, inputMsg: WebhookMessage) -> str:
        if settings.get("webhook_default_url", "") != "":
            return settings.get("webhook_default_url")
        elif inputMsg.OperationWebhook is not None and inputMsg.OperationWebhook != "":
            return inputMsg.OperationWebhook
        elif self.webhook_url is not None:
            return self.webhook_url
        else:
            logger.error("")
            return ""

    def getWebhookChannel(self, inputMsg: WebhookMessage) -> str:
        if inputMsg.Action == mythic_container.WEBHOOK_TYPE_NEW_CALLBACK and settings.get(
                "webhook_default_callback_channel", "") != "":
            return settings.get("webhook_default_callback_channel")
        elif inputMsg.Action == mythic_container.WEBHOOK_TYPE_NEW_FEEDBACK and settings.get(
                "webhook_default_feedback_channel", "") != "":
            return settings.get("webhook_default_feedback_channel")
        elif inputMsg.Action == mythic_container.WEBHOOK_TYPE_NEW_STARTUP and settings.get(
                "webhook_default_startup_channel", "") != "":
            return settings.get("webhook_default_startup_channel")
        elif inputMsg.Action == mythic_container.WEBHOOK_TYPE_NEW_ALERT and settings.get(
                "webhook_default_alert_channel", "") != "":
            return settings.get("webhook_default_alert_channel")
        elif inputMsg.Action == mythic_container.WEBHOOK_TYPE_NEW_CUSTOM and settings.get(
                "webhook_default_custom_channel", "") != "":
            return settings.get("webhook_default_custom_channel")
        elif settings.get("webhook_default_channel", "") != "":
            return settings.get("webhook_default_channel")
        elif inputMsg.OperationChannel is not None and inputMsg.OperationChannel != "":
            return inputMsg.OperationChannel
        elif self.webhook_channel is not None:
            return self.webhook_channel
        else:
            logger.error(f"No webhook channel found")
            return ""

    async def on_container_start(self, message: ContainerOnStartMessage) -> ContainerOnStartMessageResponse:
        return ContainerOnStartMessageResponse(ContainerName=self.name)

    def get_sync_message(self):
        subscriptions = []
        if self.new_alert is not None:
            subscriptions.append("new_alert")
        if self.new_feedback is not None:
            subscriptions.append("new_feedback")
        if self.new_callback is not None:
            subscriptions.append("new_callback")
        if self.new_startup is not None:
            subscriptions.append("new_startup")
        if self.new_custom is not None:
            subscriptions.append("new_custom")
        return {
            "name": self.name,
            "type": "webhook",
            "description": self.description,
            "subscriptions": subscriptions
        }


async def sendWebhookMessage(contents: dict, url: str) -> (int, str):
    try:
        async with aiohttp.ClientSession() as session:
            async with session.post(url, json=contents, ssl=False) as resp:
                return resp.status, await resp.text()
    except Exception as e:
        logger.exception(f"[-] Failed to send webhook: {e}")
        return 400, f"[-] Failed to send webhook: {e}"

webhooks: dict[str, Webhook] = {}


async def SendMythicRPCSyncWebhook(webhook_name: str) -> bool:
    try:
        webhook_services = Webhook.__subclasses__()
        for cls in webhook_services:
            web = cls()
            if web.name == "":
                continue
            if web.name == webhook_name:
                webhooks.pop(webhook_name, None)
                webhooks[web.name] = web
                await mythic_container.mythic_service.syncWebhookData(web)
                return True
        return False
    except Exception as e:
        return False



import ujson
from . import AuthBase
import traceback


async def GetIDPMetadata(msg: bytes) -> bytes:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in AuthBase.authServices.items():
            if pt.name == msgDict["container_name"]:
                response = pt.get_idp_metadata(AuthBase.GetIDPMetadataMessage(**msgDict))
                return ujson.dumps(response.to_json()).encode()
    except Exception as e:
        response = AuthBase.GetIDPMetadataMessageResponse(
            Success=False,
            Error=f"Hit exception trying to call get_idp_metadata function: {traceback.format_exc()}\n{e}"
        )
        return ujson.dumps(response.to_json()).encode()


async def GetIDPRedirect(msg: bytes) -> bytes:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in AuthBase.authServices.items():
            if pt.name == msgDict["container_name"]:
                response = pt.get_idp_redirect(AuthBase.GetIDPRedirectMessage(**msgDict))
                return ujson.dumps(response.to_json()).encode()
    except Exception as e:
        response = AuthBase.GetIDPRedirectMessageResponse(
            Success=False,
            Error=f"Hit exception trying to call get_idp_redirect function: {traceback.format_exc()}\n{e}"
        )
        return ujson.dumps(response.to_json()).encode()


async def ProcessIDPResponse(msg: bytes) -> bytes:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in AuthBase.authServices.items():
            if pt.name == msgDict["container_name"]:
                response = pt.process_idp_response(AuthBase.ProcessIDPResponseMessage(**msgDict))
                return ujson.dumps(response.to_json()).encode()
    except Exception as e:
        response = AuthBase.ProcessIDPResponseMessageResponse(
            SuccessfulAuthentication=False,
            Error=f"Hit exception trying to call process_idp_response function: {traceback.format_exc()}\n{e}"
        )
        return ujson.dumps(response.to_json()).encode()


async def GetNonIDPMetadata(msg: bytes) -> bytes:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in AuthBase.authServices.items():
            if pt.name == msgDict["container_name"]:
                response = pt.get_non_idp_metadata(AuthBase.GetNonIDPMetadataMessage(**msgDict))
                return ujson.dumps(response.to_json()).encode()
    except Exception as e:
        response = AuthBase.GetNonIDPMetadataMessageResponse(
            Success=False,
            Error=f"Hit exception trying to call get_non_idp_metadata function: {traceback.format_exc()}\n{e}"
        )
        return ujson.dumps(response.to_json()).encode()


async def GetNonIDPRedirect(msg: bytes) -> bytes:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in AuthBase.authServices.items():
            if pt.name == msgDict["container_name"]:
                response = pt.get_non_idp_redirect(AuthBase.GetNonIDPRedirectMessage(**msgDict))
                return ujson.dumps(response.to_json()).encode()
    except Exception as e:
        response = AuthBase.GetNonIDPRedirectMessageResponse(
            Success=False,
            Error=f"Hit exception trying to call get_non_idp_redirect function: {traceback.format_exc()}\n{e}"
        )
        return ujson.dumps(response.to_json()).encode()


async def ProcessNonIDPResponse(msg: bytes) -> bytes:
    try:
        msgDict = ujson.loads(msg)
        for name, pt in AuthBase.authServices.items():
            if pt.name == msgDict["container_name"]:
                response = pt.process_non_idp_response(AuthBase.ProcessNonIDPResponseMessage(**msgDict))
                return ujson.dumps(response.to_json()).encode()
    except Exception as e:
        response = AuthBase.ProcessNonIDPResponseMessageResponse(
            SuccessfulAuthentication=False,
            Error=f"Hit exception trying to call process_non_idp_response function: {traceback.format_exc()}\n{e}"
        )
        return ujson.dumps(response.to_json()).encode()



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_TASK_UPDATE = "mythic_rpc_task_update"


class MythicRPCTaskUpdateMessage:
    def __init__(self,
                 TaskID: int,
                 UpdateStatus: str = None,
                 UpdateStdout: str = None,
                 UpdateStderr: str = None,
                 UpdateCommandName: str = None,
                 UpdateCompleted: bool = None,
                 **kwargs):
        self.TaskID = TaskID
        self.UpdateStatus = UpdateStatus
        self.UpdateStdout = UpdateStdout
        self.UpdateStderr = UpdateStderr
        self.UpdateCommandName = UpdateCommandName
        self.UpdateCompleted = UpdateCompleted
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "update_status": self.UpdateStatus,
            "update_stdout": self.UpdateStdout,
            "update_stderr": self.UpdateStderr,
            "update_command_name": self.UpdateCommandName,
            "update_completed": self.UpdateCompleted
        }


class MythicRPCTaskUpdateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCTaskUpdate(
        msg: MythicRPCTaskUpdateMessage) -> MythicRPCTaskUpdateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_TASK_UPDATE,
                                                                            body=msg.to_json())
    return MythicRPCTaskUpdateMessageResponse(**response)



import logging
import sys
from .config import settings
from logging.handlers import RotatingFileHandler

LOG_FORMAT = (
    "%(levelname) -4s %(asctime)s %(funcName) "
    "-3s %(lineno) -5d: %(message)s"
)
logger = logging.getLogger("mythic")


def initialize():
    settingLogLevel = settings.get("debug_level", "info")
    update_log_to_stdout(settingLogLevel)


def update_log_to_file(filename: str, maxSizeInMB: int, maxBackups: int, debugLevel: str):
    handler = RotatingFileHandler(filename,
                                  maxBytes=maxSizeInMB * 1024 * 1024,
                                  backupCount=maxBackups)
    if debugLevel == "warning":
        logger.setLevel(logging.WARNING)
        handler.setLevel(logging.WARNING)
    elif debugLevel == "info":
        logger.setLevel(logging.INFO)
        handler.setLevel(logging.INFO)
    elif debugLevel == "debug":
        logger.setLevel(logging.DEBUG)
        handler.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.WARNING)
        handler.setLevel(logging.WARNING)
    logger.addHandler(handler)
    logger.info(f"[*] Using debug level: {debugLevel}")


def update_log_to_stdout(debugLevel: str):
    settingLogLevel = debugLevel
    handler = logging.StreamHandler(sys.stdout)
    if settingLogLevel == "warning":
        logger.setLevel(logging.WARNING)
        handler.setLevel(logging.WARNING)
    elif settingLogLevel == "info":
        logger.setLevel(logging.INFO)
        handler.setLevel(logging.INFO)
    elif settingLogLevel == "debug":
        logger.setLevel(logging.DEBUG)
        handler.setLevel(logging.DEBUG)
    else:
        logger.setLevel(logging.WARNING)
        handler.setLevel(logging.WARNING)
    formatter = logging.Formatter(LOG_FORMAT)
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    logger.info(f"[*] Using debug level: {settingLogLevel}")



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_CALLBACKTOKEN_CREATE = "mythic_rpc_callbacktoken_create"

class Token:
    def __init__(self,
                 Action: str = None, # for token_create not callbacktoken_create
                 TokenID: int = None, # for token_create not callbacktoken_create
                 User: str = None,
                 Groups: str = None,
                 Privileges: str = None,
                 ThreadID: int = None,
                 ProcessID: int = None,
                 SessionID: int = None,
                 LogonSID: str = None,
                 IntegrityLevelSID: str = None,
                 Restricted: bool = None,
                 DefaultDacl: str = None,
                 Handle: int = None,
                 Capabilities: str = None,
                 AppContainerSID: str = None,
                 AppContainerNumber: int = None,
                 **kwargs):
        self.Action = Action
        self.TokenID = TokenID
        self.User = User
        self.Groups = Groups
        self.Privileges = Privileges
        self.ThreadID = ThreadID
        self.ProcessID = ProcessID
        self.SessionID = SessionID
        self.LogonSID = LogonSID
        self.IntegrityLevelSID = IntegrityLevelSID
        self.Restricted = Restricted
        self.DefaultDacl = DefaultDacl
        self.Handle = Handle
        self.Capabilities = Capabilities
        self.AppContainerSID = AppContainerSID
        self.AppContainerNumber = AppContainerNumber
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")
    def to_json(self):
        return {
            "action": self.Action,
            "token_id": self.TokenID,
            "user": self.User,
            "groups": self.Groups,
            "privileges": self.Privileges,
            "thread_id": self.ThreadID,
            "process_id": self.ProcessID,
            "session_id": self.SessionID,
            "logon_sid": self.LogonSID,
            "integrity_level_sid": self.IntegrityLevelSID,
            "restricted": self.Restricted,
            "default_dacl": self.DefaultDacl,
            "handle": self.Handle,
            "capabilities": self.Capabilities,
            "app_container_sid": self.AppContainerSID,
            "app_container_number": self.AppContainerNumber
        }
class MythicRPCCallbackTokenData:
    def __init__(self,
                 Action: str = "add",
                 Host: str = None,
                 TokenId: int = None,
                 TokenInfo: Token = None,
                 **kwargs):
        self.Host = Host
        self.Action = Action
        self.TokenInfo = TokenInfo
        self.TokenId = TokenId
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "host": self.Host,
            "action": self.Action,
            "token": self.TokenInfo.to_json() if self.TokenInfo is not None else None,
            "token_id": self.TokenId
        }


class MythicRPCCallbackTokenCreateMessage:
    def __init__(self,
                 TaskID: int,
                 CallbackTokens: list[MythicRPCCallbackTokenData],
                 **kwargs):
        self.TaskID = TaskID
        self.CallbackTokens = CallbackTokens
        for k, v in kwargs:
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "callbacktokens": [x.to_json() for x in self.CallbackTokens]
        }


class MythicRPCCallbackTokenCreateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCCallbackTokenCreate(
        msg: MythicRPCCallbackTokenCreateMessage) -> MythicRPCCallbackTokenCreateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_CALLBACKTOKEN_CREATE,
                                                                            body=msg.to_json())
    return MythicRPCCallbackTokenCreateMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger
import base64

MYTHIC_RPC_FILEBROWSER_PARSE_PATH = "mythic_rpc_filebrowser_parse_path"


class MythicRPCFileBrowserParsePathMessage:
    def __init__(self,
                 Path: str = None,
                 **kwargs):
        self.Path = Path
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "path": self.Path,
        }

class AnalyzedPath:
    def __init__(self,
                 path_pieces: [str] = [],
                 path_separator: str = "",
                 host: str = ""):
        self.PathPieces = path_pieces
        self.PathSeparator = path_separator
        self.Host = host

    def to_json(self):
        return {
            "path_pieces": self.PathPieces,
            "path_separator": self.PathSeparator,
            "host": self.Host
        }
class MythicRPCFileBrowserParsePathMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 analyzed_path: dict = {},
                 **kwargs):
        self.Success = success
        self.Error = error
        self.AnalyzedPath = AnalyzedPath(**analyzed_path)
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "analyzed_path": self.AnalyzedPath.to_json(),
        }


async def SendMythicRPCFileBrowserParsePath(
        msg: MythicRPCFileBrowserParsePathMessage) -> MythicRPCFileBrowserParsePathMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(
        queue=MYTHIC_RPC_FILEBROWSER_PARSE_PATH,
        body=msg.to_json())
    return MythicRPCFileBrowserParsePathMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_CALLBACK_UPDATE = "mythic_rpc_callback_update"


class MythicRPCCallbackUpdateMessage:
    def __init__(self,
                 AgentCallbackUUID: str = None,
                 CallbackID: int = None,
                 TaskID: int = None,
                 EncryptionKeyBase64: str = None,
                 DecryptionKeyBase64: str = None,
                 CryptoType: str = None,
                 User: str = None,
                 Host: str = None,
                 PID: int = None,
                 ExtraInfo: str = None,
                 SleepInfo: str = None,
                 IP: str = None,
                 ExternalIP: str = None,
                 IntegrityLevel: int = None,
                 OS: str = None,
                 Domain: str = None,
                 Architecture: str = None,
                 Description: str = None,
                 ProcessName: str = None,
                 UpdateLastCheckinTime: bool = False,
                 UpdateLastCheckinTimeViaC2Profile: str = None,
                 **kwargs):
        self.AgentCallbackUUID = AgentCallbackUUID
        self.CallbackID = CallbackID
        self.TaskID = TaskID
        self.EncryptionKeyBase64 = EncryptionKeyBase64
        self.DecryptionKeyBase64 = DecryptionKeyBase64
        self.CryptoType = CryptoType
        self.User = User
        self.Host = Host
        self.PID = PID
        self.ExtraInfo = ExtraInfo
        self.SleepInfo = SleepInfo
        self.Ip = IP
        self.ExternalIP = ExternalIP
        self.IntegrityLevel = IntegrityLevel
        self.Os = OS
        self.Domain = Domain
        self.Architecture = Architecture
        self.Description = Description
        self.ProcessName = ProcessName
        self.UpdateLastCheckinTime = UpdateLastCheckinTime
        self.UpdateLastCheckinTimeViaC2Profile = UpdateLastCheckinTimeViaC2Profile
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "agent_callback_id": self.AgentCallbackUUID,
            "callback_id": self.CallbackID,
            "task_id": self.TaskID,
            "encryption_key": self.EncryptionKeyBase64,
            "decryption_key": self.DecryptionKeyBase64,
            "crypto_type": self.CryptoType,
            "user": self.User,
            "host": self.Host,
            "pid": self.PID,
            "extra_info": self.ExtraInfo,
            "sleep_info": self.SleepInfo,
            "ip": self.Ip,
            "external_ip": self.ExternalIP,
            "integrity_level": self.IntegrityLevel,
            "os": self.Os,
            "domain": self.Domain,
            "architecture": self.Architecture,
            "description": self.Description,
            "process_name": self.ProcessName,
            "update_last_checkin_time": self.UpdateLastCheckinTime,
            "update_last_checkin_time_via_c2_profile": self.UpdateLastCheckinTimeViaC2Profile
        }


class MythicRPCCallbackUpdateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCCallbackUpdate(
        msg: MythicRPCCallbackUpdateMessage) -> MythicRPCCallbackUpdateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_CALLBACK_UPDATE,
                                                                            body=msg.to_json())
    return MythicRPCCallbackUpdateMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger
from mythic_container.utils_mythic_file_transfer import sendFileToMythic

MYTHIC_RPC_FILE_UPDATE = "mythic_rpc_file_update"


class MythicRPCFileUpdateMessage:
    def __init__(self,
                 AgentFileID: str,
                 Comment: str = None,
                 Filename: str = None,
                 AppendContents: bytes = None,
                 ReplaceContents: bytes = None,
                 Delete: bool = None,
                 DeleteAfterFetch: bool = None,
                 **kwargs):
        self.AgentFileID = AgentFileID
        self.Filename = Filename
        self.Comment = Comment
        self.AppendContents = AppendContents
        self.ReplaceContents = ReplaceContents
        self.Delete = Delete
        self.DeleteAfterFetch = DeleteAfterFetch
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "file_id": self.AgentFileID,
            "filename": self.Filename,
            "comment": self.Comment,
            "append_contents": self.AppendContents,
            "delete": self.Delete,
            "delete_after_fetch": self.DeleteAfterFetch
        }


class MythicRPCFileUpdateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCFileUpdate(
        msg: MythicRPCFileUpdateMessage) -> MythicRPCFileUpdateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_FILE_UPDATE,
                                                                            body=msg.to_json())
    finalResponse = MythicRPCFileUpdateMessageResponse(**response)
    if finalResponse.Success and msg.ReplaceContents is not None:
        # we need to ship off the file contents now
        if await sendFileToMythic(msg.ReplaceContents, msg.AgentFileID):
            return finalResponse
        else:
            finalResponse.Success = False
            finalResponse.Error = "Failed to upload file contents to Mythic over HTTP"
            return finalResponse
    return finalResponse



import mythic_container
from mythic_container.logging import logger
from .send_mythic_rpc_callbacktoken_create import MythicRPCCallbackTokenData

MYTHIC_RPC_CALLBACKTOKEN_REMOVE = "mythic_rpc_callbacktoken_remove"

class MythicRPCCallbackTokenRemoveMessage:
    def __init__(self,
                 TaskID: int,
                 CallbackTokens: list[MythicRPCCallbackTokenData],
                 **kwargs):
        self.TaskID = TaskID
        self.CallbackTokens = CallbackTokens
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "callbacktokens": [x.to_json() for x in self.CallbackTokens]
        }


class MythicRPCCallbackTokenRemoveMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCCallbackTokenRemove(
        msg: MythicRPCCallbackTokenRemoveMessage) -> MythicRPCCallbackTokenRemoveMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_CALLBACKTOKEN_REMOVE,
                                                                            body=msg.to_json())
    return MythicRPCCallbackTokenRemoveMessageResponse(**response)



from enum import Enum
import base64

import mythic_container
from .MythicCommandBase import PTOnNewCallbackAllData, PTOnNewCallbackResponse, PTCheckIfCallbacksAliveMessage, \
    PTCheckIfCallbacksAliveMessageResponse
from .logging import logger
import json
from collections.abc import Callable, Awaitable
import pathlib
from .SharedClasses import ContainerOnStartMessage, ContainerOnStartMessageResponse


class BuildStatus:
    """Build Status

    This can either be BuildStatus.Success or BuildStatus.Error

    Attributes:
        Success (str): Successful build
        Error (str): Failed build
    """
    Success = "success"
    Error = "error"


class SupportedOS:
    """Supported Operating System

    This OS value is selected first when generating a payload or wrapper.

    If you don't want to use a listed value, supply your own with
    SupportedOS("my os")
    """
    Windows = "Windows"
    MacOS = "macOS"
    Linux = "Linux"
    WebShell = "WebShell"
    Chrome = "Chrome"

    def __init__(self, os: str):
        self.os = os

    def __str__(self):
        return self.os


class AgentType:
    """Type of agent container

    This identifies what kind of container this is and how it shows up in the Mythic UI.

    Agent - your normal payload type
    Wrapper - identifies that this payload type doesn't use C2 and wraps an existing payload
    Service - this payload type interacts with an external service
    CommandAugment - this payload type defines commands that are injected into existing callbacks

    """
    Agent = "agent"
    Wrapper = "wrapper"
    Service = "service"
    CommandAugment = "command_augment"


class MessageFormat:
    """Message format of the agent

    Mythic natively supports both JSON and XML messages. Anything else and you'll need a translation container

    """
    JSON = "json"
    XML = "xml"


class BuildParameterType(str, Enum):
    """Types of parameters available for building payloads

    Attributes:
        String:
            A string value
        ChooseOne:
            A list of choices for the user to select exactly one
        ChooseMultiple:
            A list of choices for the user to select 0 or more
        ChooseOneCustom:
            A list of choices for the user to select exactly one OR write in their own
        Array:
            The user can supply multiple values in an Array format
        Date:
            The user can select a Date in YYYY-MM-DD format
        Dictionary:
            The user can supply a dictionary of values
        Boolean:
            The user can toggle a switch for True/False
        File:
            The user can select a file that gets uploaded - a file UUID gets passed in during build
        FileMultiple:
            The user can select multiple files that get uploaded - an array of file UUIDs gets passed in during build
        TypedArray:
            The user can supply an array where each element also has a drop-down option of choices
    """
    String = "String"
    ChooseOne = "ChooseOne"
    ChooseOneCustom = "ChooseOneCustom"
    ChooseMultiple = "ChooseMultiple"
    Array = "Array"
    Date = "Date"
    Dictionary = "Dictionary"
    Boolean = "Boolean"
    File = "File"
    FileMultiple = "FileMultiple"
    TypedArray = "TypedArray"
    Number = "Number"


class DictionaryChoice:
    """A single dictionary choice/option for the UI when select C2 Profile Parameters

    Attributes:
        name (str): Name of the choice
        default_show (bool): Should this be pre-selected with a default value
        default_value (str): Default value to fill in

    """

    def __init__(self,
                 name: str,
                 default_value: str = "",
                 default_show: bool = True):
        self.name = name
        self.default_show = default_show
        self.default_value = default_value

    def to_json(self):
        return {
            "name": self.name,
            "default_value": self.default_value,
            "default_show": self.default_show
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class BuildParameter:
    """Build Parameter Definition for use when generating payloads

    Attributes:
        name (str):
            Name of the parameter for scripting and for when building payloads
        description (str):
            Informative description displayed when building a payload
        default_value (any):
            Default value to pre-populate
        randomize (bool):
            Should this value be randomized (requires format_string)
        format_string (str):
            A regex used for randomizing values if randomize is true
        parameter_type (BuildParameterType):
            The type of parameter this is
        required (bool):
            Is this parameter required to have a non-empty value or not
        verifier_regex (str):
            Regex used to verify that the user typed something appropriate
        choices (list[str]):
            Choices for ChooseOne parameter type
        dictionary_choices (list[DictionaryChoice]):
            Configuration options for the Dictionary parameter type
        crypto_type (bool):
            Indicate if this value should be used to generate a crypto key or not

    """

    def __init__(
            self,
            name: str,
            parameter_type: BuildParameterType = None,
            description: str = None,
            required: bool = None,
            randomize: bool = None,
            format_string: str = "",
            crypto_type: bool = False,
            verifier_regex: str = None,
            default_value: any = None,
            choices: list[str] = None,
            dictionary_choices: list[DictionaryChoice] = None,
            value: any = None,
            verifier_func: callable = None,
    ):
        self.name = name
        self.verifier_func = verifier_func
        self.parameter_type = (
            parameter_type if parameter_type is not None else BuildParameterType.String
        )
        self.description = description if description is not None else ""
        self.required = required if required is not None else True
        self.verifier_regex = verifier_regex if verifier_regex is not None else ""
        self.default_value = default_value
        if value is None:
            self.value = default_value
        else:
            self.value = value
        self.choices = choices
        self.dictionary_choices = dictionary_choices
        self.crypto_type = crypto_type
        self.randomize = randomize
        self.format_string = format_string

    def to_json(self):
        return {
            "name": self.name,
            "description": self.description,
            "default_value": self.default_value,
            "randomize": self.randomize,
            "format_string": self.format_string,
            "required": self.required,
            "parameter_type": self.parameter_type.value,
            "verifier_regex": self.verifier_regex,
            "crypto_type": self.crypto_type,
            "choices": self.choices,
            "dictionary_choices": [x.to_json() for x in
                                   self.dictionary_choices] if self.dictionary_choices is not None else None
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2ProfileParameters:
    """C2 Profile information for building a payload

    Functions:
        get_c2profile:
            Get a dictionary with metadata about the c2 profile (name, is_p2p)
        get_parameters_dict:
            Get a dictionary of the parameters for the profile
    """

    def __init__(self, c2profile: dict, parameters: dict = None):
        self.parameters = {}
        self.c2profile = c2profile
        if parameters is not None:
            self.parameters = parameters

    def get_parameters_dict(self):
        return self.parameters

    def get_c2profile(self):
        return self.c2profile


class CommandList:
    """List of commands that the user wants to build into an agent

    This might include script_only commands that are available in the UI but not actual commands with files for your agent.

    Attributes:
        commands (list[str]):
            The names of the commands

   Functions:
       get_commands:
           Get a list of the command names
   """

    def __init__(self, commands: [str] = None):
        self.commands = []
        if commands is not None:
            self.commands = commands

    def get_commands(self) -> [str]:
        return self.commands

    def remove_command(self, command: str):
        self.commands.remove(command)

    def add_command(self, command: str):
        for c in self.commands:
            if c == command:
                return
        self.commands.append(command)

    def clear(self):
        self.commands = []


class BuildResponse:
    """The result of attempting to build a payload

    Attributes:
        status (BuildStatus):
            The status of the build
        payload (bytes):
            The final payload bytes to send back to Mythic if any
        build_message (str):
            A build message to show to the user
        build_stderr (str):
            Any stderr messages you want to save as part of the build process (also shown to the user in case of error)
        build_stdout (str):
            Any stdout data you want to save as part of the build process
        updated_command_list (list[str]):
            An updated list of commands that are actually being built into the payload. This is handy if the user selected commandA but you aren't building it into the payload for some reason. Similarly, it's helpful if the user selected commandA but you also need commandB for that to work, so you can reflect that change back to Mythic.
        updated_filename (str):
            An updated filename - this is useful if the user selects an option during build that changes the file type (ex: exe, dll, zip) and you want to reflect that back in the filename for easier downloading.
   Functions:
       get_commands:
           Get a list of the command names
   """

    def __init__(self, status: BuildStatus, payload: bytes = None, build_message: str = None, build_stderr: str = None,
                 build_stdout: str = None, updated_command_list: [str] = None, updated_filename: str = None):
        self.status = status
        self.payload = payload if payload is not None else b""
        self.build_message = build_message if build_message is not None else ""
        self.build_stderr = build_stderr if build_stderr is not None else ""
        self.build_stdout = build_stdout if build_stdout is not None else ""
        self.updated_command_list = updated_command_list
        self.updated_filename = updated_filename

    def get_status(self) -> BuildStatus:
        return self.status

    def set_status(self, status: BuildStatus):
        self.status = status

    def get_payload(self) -> bytes:
        return self.payload

    def set_payload(self, payload: bytes):
        self.payload = payload

    def set_build_message(self, build_message: str):
        self.build_message = build_message

    def get_build_message(self) -> str:
        return self.build_message

    def set_build_stderr(self, build_stderr: str):
        self.build_stderr = build_stderr

    def get_build_stderr(self) -> str:
        return self.build_stderr

    def get_build_stdout(self) -> str:
        return self.build_stdout

    def set_build_stdout(self, build_stdout: str):
        self.build_stdout = build_stdout

    def get_updated_command_list(self):
        return self.updated_command_list

    def set_updated_command_list(self, command_list):
        self.updated_command_list = command_list

    def get_updated_filename(self):
        return self.updated_filename

    def set_updated_filename(self, filename: str):
        self.updated_filename = filename


class BuildStep:
    """A tracked step in the build process for a payload type

    Attributes:
        step_name (str):
            The name of the step to display to users
        step_description (str):
            The description of the step to display to users

   """

    def __init__(self,
                 step_name: str,
                 step_description: str):
        self.step_name = step_name
        self.step_description = step_description

    def to_json(self):
        return {
            "step_name": self.step_name,
            "step_description": self.step_description
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class PTOtherServiceRPCMessage:
    """Request to call an RPC function of another C2 Profile or Payload Type

    Attributes:
        ServiceName (str): Name of the C2 Profile or Payload Type
        ServiceRPCFunction (str): Name of the function to call
        ServiceRPCFunctionArguments (dict): Arguments to that function

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 ServiceName: str = None,
                 service_name: str = None,
                 ServiceRPCFunction: str = None,
                 service_function: str = None,
                 ServiceRPCFunctionArguments: dict = None,
                 service_arguments: dict = None,
                 **kwargs):
        self.ServiceName = ServiceName
        if self.ServiceName is None:
            self.ServiceName = service_name
        self.ServiceRPCFunction = ServiceRPCFunction
        if self.ServiceRPCFunction is None:
            self.ServiceRPCFunction = service_function
        self.ServiceRPCFunctionArguments = ServiceRPCFunctionArguments
        if self.ServiceRPCFunctionArguments is None:
            self.ServiceRPCFunctionArguments = service_arguments
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "service_name": self.ServiceName,
            "service_function": self.ServiceRPCFunction,
            "service_arguments": self.ServiceRPCFunctionArguments
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class PTOtherServiceRPCMessageResponse:
    """Result of running an RPC call from another service

    Attributes:
        Success (bool): Did the RPC succeed or fail
        Error (str): Error message if the RPC check failed
        Result (dict): Result from the RPC

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 success: bool = None,
                 error: str = None,
                 result: dict = None,
                 Success: bool = None,
                 Error: str = None,
                 Result: dict = None,
                 **kwargs):
        self.Success = Success
        if self.Success is None:
            self.Success = success
        self.Error = Error
        if self.Error is None:
            self.Error = error
        self.Result = Result
        if self.Result is None:
            self.Result = result
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "result": self.Result
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class PayloadType:
    """Payload Type to import and sync with Mythic

    Attributes:
        name (str):
            Name of the payload type
        file_extension (str):
            Default file extension to apply to payloads
        author (str):
            Author of the Payload Type
        supported_os (list[SupportedOS]):
            List of Operating Systems that this Payload Type supports
        wrapper (bool):
            Is this payload type a wrapper (i.e. it takes in other payload types and wraps them up in a new format)
        wrapped_payloads (list[str]):
            What wrappers does this payload type support (ex: service_wrapper)
        note (str):
            A description of the payload type to present to users (legacy, still works)
        description (str):
            A description of the payload type to present to users (use this instead of note)
        supports_dynamic_loading (bool):
            Does this payload type support dynamically choosing which commands to build in or not. If this is `False` then when building a payload for this payload type, you won't get the option to pick which commands to add to the payload - they'll all automatically get added.
        c2_profiles (list[str]):
            List of C2 Profile names that this Payload Type supports
        build_parameters (list[BuildParameter]):
            List of build parameters for this Payload Type that the user can modify when building the payload
        message_uuid_length (int):
            The length of UUIDs you will be using in agent messages, can be either 36 (string) or 16 (bytes)
        command_augment_supported_agents (list[str]):
            If this is an AgentType.CommandAugment container, then you can specify the names of the payload types where you want these commands injected specifically
        uuid (str):
            UUID of the payload
        filename (str):
            The original filename of the payload that the user selected
        c2info (list[C2ProfileParameters]):
            List of C2 Profiles that the user selected to build into the agent along with their parameter values
        commands (CommandList):
            List of commands the user selected to build into the agent
        wrapped_payload (bytes):
            If this is a wrapper payload type, this is the raw bytes of the payload to wrap
        wrapped_payload_uuid (str):
            If this is a wrapper payload type, this is the UUID of the payload to wrap
        selected_os (str):
            The OS the user selected for the first step when building this payload
        build_steps (list[BuildStep]):
            A list of build steps that will be taken during the build process. As you build, you can report back the status of each step so that the user knows what's going on.
        agent_icon_path (str):
            Path to the agent icon you want to use with Mythic. This MUST be a .svg file.
        agent_icon_bytes (bytes):
            If you don't want to provide the path, you can optionally provide the raw bytes to the .svg file here.
        dark_mode_agent_icon_path (str):
            Path to the agent icon dark mode you want to use with Mythic. This MUST be a .svg file.
        dark_mode_agent_icon_bytes (bytes):
            If you don't want to provide the path, you can optionally provide the raw bytes to the dark mode .svg file here.
        translation_container (str):
            If your payload type uses a translation container, provide the name of it here
        mythic_encrypts (bool):
            Indicate if Mythic should handle encryption/decryption for you or if you want a translation container to do it instead. This is often useful if you want to have a separate form of encryption than what Mythic provides.
        agent_path (Path):
            The path to your agent's main folder. This is used to help determine paths to your code and browser scripts if they aren't provided
        agent_code_path (Path):
            The path to your agent's actual source code
        agent_browserscript_path (Path):
            The path to the folder holding your browser scripts so that they can be fetched when Syncing
        custom_rpc_functions (dict):
            Dictionary of RPC name to awaitable RPC function that other services can call
        message_format (str):
            Defaults to `json`, but you can optionally specify `xml` to have your messages parsed and returned as xml instead of JSON for Mythic
        agent_type (str):
            Defaults to `agent`, but you can optionally specify `service` to report back as a 3rd party service agent
        use_display_params_for_cli_history (bool):
            Defaults to False, but this dictates if the original params (potentially JSON) or the task's display params are used when doing up/down arrows through history on the command line

    Functions:
        build(self):
            Given an instance of a bare payload and all the configuration options that the user selected (build parameters, c2 profile parameters), build the payload
        get_parameter:
            Get the value of a build parameter
        on_container_start(self, ContainerOnStartMessage):
            Given some configuration information about each operation, you have the option to configure thigns as needed within your container
        on_new_callback(self, newCallback):
            Given the context of a new callback, perform some initial analysis and tasking
        check_if_callbacks_alive(self, callbacks):
            Given info about active callbacks based on this payload type, identify if they are potentially dead or not based on their last checkin time and sleep info
    """
    name: str = ""
    file_extension: str = ""
    author: str = ""
    supported_os: [str] = []
    wrapper: bool = False
    wrapped_payloads: [str] = []
    note: str = ""
    description: str = ""
    supports_dynamic_loading: bool = False
    c2_profiles: [str] = []
    build_parameters: [BuildParameter] = []
    message_uuid_length: int = 36
    command_augment_supported_agents: [str] = []
    build_steps = []
    agent_icon_path: str = None
    agent_icon_bytes: bytes = None
    dark_mode_agent_icon_path: str = None
    dark_mode_agent_icon_bytes: bytes = None
    translation_container = None
    mythic_encrypts = True
    agent_path = None
    agent_code_path = None
    agent_browserscript_path = None
    message_format: MessageFormat = MessageFormat.JSON
    agent_type: AgentType = AgentType.Agent
    use_display_params_for_cli_history = False
    custom_rpc_functions: dict[
        str, Callable[[PTOtherServiceRPCMessage], Awaitable[PTOtherServiceRPCMessageResponse]]] = {}

    # These following fields are established when the build function is called
    uuid: str = None
    c2info: [C2ProfileParameters] = None
    commands: CommandList = None
    wrapped_payload_uuid: str = None
    wrapped_payload: bytes = None
    selected_os: str = None
    filename: str = None

    def __init__(
            self,
            uuid: str = None,
            filename: str = "",
            c2info: [C2ProfileParameters] = None,
            selected_os: str = None,
            commands: CommandList = None,
            wrapped_payload_uuid: str = None,
            wrapped_payload: bytes = None,
            secrets: dict = {},
    ):
        self.uuid = uuid
        self.c2info = c2info
        self.selected_os = selected_os
        self.commands = commands
        self.filename = filename
        self.wrapped_payload = wrapped_payload
        self.wrapped_payload_uuid = wrapped_payload_uuid
        self.secrets = secrets
        if self.agent_path is None:
            self.agent_path = pathlib.Path(".") / self.name
            logger.error(f"{self.name} has no agent_path set, setting to {self.agent_path}")
        if self.agent_code_path is None:
            self.agent_code_path = self.agent_path / "agent_code"
            logger.error(f"{self.name} has no agent_code_path set, setting to {self.agent_code_path}")
        if self.agent_browserscript_path is None:
            self.agent_browserscript_path = self.agent_path / "browser_scripts"

    async def build(self) -> BuildResponse:
        pass

    async def on_new_callback(self, newCallback: PTOnNewCallbackAllData) -> PTOnNewCallbackResponse:
        return PTOnNewCallbackResponse(AgentCallbackID=newCallback.Callback.AgentCallbackID, Success=True)

    async def check_if_callbacks_alive(self,
                                       message: PTCheckIfCallbacksAliveMessage) -> PTCheckIfCallbacksAliveMessageResponse:
        return PTCheckIfCallbacksAliveMessageResponse(Success=True)

    async def on_container_start(self, message: ContainerOnStartMessage) -> ContainerOnStartMessageResponse:
        return ContainerOnStartMessageResponse(ContainerName=self.name)

    def get_parameter(self, name):
        for arg in self.build_parameters:
            if arg.name == name:
                return arg.value
        return None

    async def set_and_validate_build_parameters(self, buildinfo: dict):
        # set values for all of the key-value pairs presented to us
        for bp in self.build_parameters:
            if bp.name in buildinfo and buildinfo[bp.name] is not None:
                bp.value = buildinfo[bp.name]
            if bp.required and bp.value is None:
                raise ValueError(
                    "{} is a required parameter but has no value".format(bp.name)
                )

    def get_build_instance_values(self):
        values = {}
        for bp in self.build_parameters:
            if bp.value is not None:
                values[bp.name] = bp.value
        return values

    def to_json(self):
        agent_bytes = self.agent_icon_bytes
        dark_mode_agent_bytes = self.dark_mode_agent_icon_bytes
        self.name = self.name.lower()
        if agent_bytes is None:
            if self.agent_icon_path is not None:
                # read agent icon path
                try:
                    with open(self.agent_icon_path, "rb") as f:
                        agent_bytes = f.read()
                except Exception as e:
                    logger.exception(f"failed to read agent icon from ({self.agent_icon_path}): {e}")
                    agent_bytes = b""
            else:
                logger.error(f"{self.name} has no agent_icon_bytes or agent_icon_path specified, no icon will be used")
                agent_bytes = b""
        if dark_mode_agent_bytes is None:
            if self.dark_mode_agent_icon_path is not None:
                try:
                    with open(self.dark_mode_agent_icon_path, "rb") as f:
                        dark_mode_agent_bytes = f.read()
                except Exception as e:
                    logger.exception(
                        f"failed to read dark mode agent icon from ({self.dark_mode_agent_icon_path}): {e}")
                    dark_mode_agent_bytes = b""
            else:
                dark_mode_agent_bytes = agent_bytes
        if self.agent_type == AgentType.Wrapper:
            self.wrapper = True
        elif self.wrapper:
            self.agent_type = AgentType.Wrapper
        desc = self.description
        if desc == "":
            desc = self.note
        return {
            "name": self.name,
            "file_extension": self.file_extension,
            "author": self.author,
            "supported_os": [str(x) for x in self.supported_os],
            "wrapper": self.wrapper,
            "supported_wrapper_payload_types": self.wrapped_payloads,
            "supports_dynamic_load": self.supports_dynamic_loading,
            "description": desc,
            "build_parameters": [b.to_json() for b in self.build_parameters],
            "supported_c2_profiles": self.c2_profiles,
            "translation_container_name": self.translation_container,
            "mythic_encrypts": self.mythic_encrypts,
            "build_steps": [x.to_json() for x in self.build_steps],
            "agent_icon": base64.b64encode(agent_bytes).decode(),
            "dark_mode_agent_icon": base64.b64encode(dark_mode_agent_bytes).decode(),
            "message_format": self.message_format,
            "agent_type": self.agent_type,
            "message_uuid_length": self.message_uuid_length,
            "command_augment_supported_agents": self.command_augment_supported_agents,
            "use_display_params_for_cli_history": self.use_display_params_for_cli_history,
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


payloadTypes: dict[str, PayloadType] = {}


async def SendMythicRPCSyncPayloadType(payload_type: str, extraCommands: [mythic_container.MythicCommandBase.CommandBase]) -> bool:
    try:
        for name, pt in payloadTypes.items():
            if pt.name == payload_type:
                mythic_container.MythicCommandBase.commands.pop(pt.name, None)
                await mythic_container.mythic_service.syncPayloadData(pt, extraCommands, True)
                return True
        return False
    except Exception as e:
        logger.exception(f"Failed to re-sync payload type: {e}")
        return False



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_FILEBROWSER_CREATE = "mythic_rpc_filebrowser_create"

class MythicRPCFileBrowserDataChildren:
    def __init__(self,
                 IsFile: bool = None,
                 Permissions: dict = None,
                 Name: str = None,
                 AccessTime: int = None,
                 ModifyTime: int = None,
                 Size: int = None,
                 **kwargs):
        self.IsFile = IsFile
        self.Permissions = Permissions
        self.Name = Name
        self.AccessTime = AccessTime
        self.ModifyTime = ModifyTime
        self.Size = Size
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")
    def to_json(self):
        return {
            "is_file": self.IsFile,
            "permissions": self.Permissions,
            "name": self.Name,
            "access_time": self.AccessTime,
            "modify_time": self.ModifyTime,
            "size": self.Size
        }
class MythicRPCFileBrowserData:
    def __init__(self,
                 Host: str = None,
                 IsFile: bool = None,
                 Permissions: dict = None,
                 Name: str = None,
                 ParentPath: str = None,
                 Success: bool = None,
                 AccessTime: int = None,
                 ModifyTime: int = None,
                 Size: int = None,
                 UpdateDeleted: bool = None,
                 Files: list[MythicRPCFileBrowserDataChildren] = None,
                 **kwargs):
        self.Host = Host
        self.IsFile = IsFile
        self.Permissions = Permissions
        self.Name = Name
        self.ParentPath = ParentPath
        self.Success = Success
        self.AccessTime = AccessTime
        self.ModifyTime = ModifyTime
        self.Size = Size
        self.UpdateDeleted = UpdateDeleted
        self.Files = Files
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "host": self.Host,
            "is_file": self.IsFile,
            "permissions": self.Permissions,
            "name": self.Name,
            "parent_path": self.ParentPath,
            "success": self.Success,
            "access_time": self.AccessTime,
            "modify_time": self.ModifyTime,
            "size": self.Size,
            "update_deleted": self.UpdateDeleted,
            "files": [x.to_json() for x in self.Files]
        }


class MythicRPCFileBrowserCreateMessage:
    def __init__(self,
                 TaskID: int,
                 FileBrowser: MythicRPCFileBrowserData,
                 **kwargs):
        self.TaskID = TaskID
        self.FileBrowser = FileBrowser
        for k, v in kwargs:
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "filebrowser": self.FileBrowser.to_json()
        }


class MythicRPCFileBrowserCreateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCFileBrowserCreate(
        msg: MythicRPCFileBrowserCreateMessage) -> MythicRPCFileBrowserCreateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_FILEBROWSER_CREATE,
                                                                            body=msg.to_json())
    return MythicRPCFileBrowserCreateMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_PAYLOAD_CREATE_FROM_SCRATCH = "mythic_rpc_payload_create_from_scratch"


class MythicRPCPayloadConfigurationC2Profile:
    def __init__(self,
                 Name: str = None,
                 Parameters: dict = None,
                 c2_profile: str = None,
                 c2_profile_is_p2p: bool = False,
                 c2_profile_parameters: dict = None,
                 **kwargs):
        self.Name = Name
        self.Parameters = Parameters
        self.IsP2P = c2_profile_is_p2p
        if c2_profile is not None:
            self.Name = c2_profile
        if c2_profile_parameters is not None:
            self.Parameters = c2_profile_parameters
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "c2_profile": self.Name,
            "c2_profile_parameters": self.Parameters,
            "c2_profile_is_p2p": self.IsP2P
        }


class MythicRPCPayloadConfigurationBuildParameter:
    def __init__(self,
                 Name: str = None,
                 Value: any = None,
                 name: str = None,
                 value: any = None,
                 **kwargs):
        self.Name = Name
        self.Value = Value
        if name is not None:
            self.Name = name
        if value is not None:
            self.Value = value
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "name": self.Name,
            "value": self.Value
        }


class MythicRPCPayloadConfiguration:
    def __init__(self,
                 Description: str = None,
                 PayloadType: str = None,
                 C2Profiles: list[MythicRPCPayloadConfigurationC2Profile] = None,
                 BuildParameters: list[MythicRPCPayloadConfigurationBuildParameter] = None,
                 Commands: list[str] = None,
                 SelectedOS: str = None,
                 Filename: str = None,
                 WrappedPayloadUUID: str = None,
                 UUID: str = None,
                 AgentFileId: str = None,
                 **kwargs):
        self.Description = Description
        self.PayloadType = PayloadType
        self.C2Profiles = C2Profiles
        self.BuildParameters = BuildParameters
        self.Commands = Commands
        self.SelectedOS = SelectedOS
        self.Filename = Filename
        self.WrappedPayloadUUID = WrappedPayloadUUID
        self.UUID = UUID
        self.AgentFileId = AgentFileId
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "description": self.Description,
            "payload_type": self.PayloadType,
            "c2_profiles": [x.to_json() for x in self.C2Profiles],
            "build_parameters": [x.to_json() for x in self.BuildParameters],
            "commands": self.Commands,
            "selected_os": self.SelectedOS,
            "filename": self.Filename,
            "wrapped_payload": self.WrappedPayloadUUID,
            "uuid": self.UUID,
            "agent_file_id": self.AgentFileId
        }


class MythicRPCPayloadCreateFromScratchMessage:
    def __init__(self,
                 TaskID: int,
                 PayloadConfiguration: MythicRPCPayloadConfiguration,
                 RemoteHost: str = None,
                 **kwargs):
        self.TaskID = TaskID
        self.PayloadConfiguration = PayloadConfiguration
        self.RemoteHost = RemoteHost
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "payload_configuration": self.PayloadConfiguration.to_json(),
            "remote_host": self.RemoteHost
        }


class MythicRPCPayloadCreateFromScratchMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 new_payload_uuid: str = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.NewPayloadUUID = new_payload_uuid
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCPayloadCreateFromScratch(
        msg: MythicRPCPayloadCreateFromScratchMessage) -> MythicRPCPayloadCreateFromScratchMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(
        queue=MYTHIC_RPC_PAYLOAD_CREATE_FROM_SCRATCH,
        body=msg.to_json())

    return MythicRPCPayloadCreateFromScratchMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger
from mythic_container.utils_mythic_file_transfer import getFileFromMythic

MYTHIC_RPC_PAYLOADONHOST_CREATE = "mythic_rpc_payloadonhost_create"


class MythicRPCPayloadOnHostCreateData:
    def __init__(self,
                 Host: str = None,
                 PayloadId: int = None,
                 PayloadUUID: str = None,
                 **kwargs):
        self.Host = Host
        self.PayloadId = PayloadId
        self.PayloadUUID = PayloadUUID
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")
    def to_json(self):
        return {
            "host": self.Host,
            "payload_id": self.PayloadId,
            "payload_uuid": self.PayloadUUID
        }

class MythicRPCPayloadOnHostCreateMessage:
    def __init__(self,
                 TaskID: int,
                 PayloadOnHost: MythicRPCPayloadOnHostCreateData = None,
                 **kwargs):
        self.TaskID = TaskID
        self.PayloadOnHost = PayloadOnHost
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "payload_on_host": self.PayloadOnHost
        }


class MythicRPCPayloadOnHostCreateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCPayloadOnHostCreate(
        msg: MythicRPCPayloadOnHostCreateMessage) -> MythicRPCPayloadOnHostCreateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_PAYLOADONHOST_CREATE,
                                                                            body=msg.to_json())
    return MythicRPCPayloadOnHostCreateMessageResponse(**response)



import pathlib
from enum import Enum
import json
from collections.abc import Awaitable, Callable
from .logging import logger
import sys
from typing import List
from .SharedClasses import ContainerOnStartMessage, ContainerOnStartMessageResponse


class C2OPSECMessage:
    """Payload's C2 Profile configuration for OPSEC checking

    Attributes:
        Name (str): Name of the C2 Profile
        Parameters (dict): Dictionary of C2 Parameter name:value

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 c2_profile_name: str,
                 parameters: dict,
                 **kwargs):
        self.Name = c2_profile_name
        self.Parameters = parameters
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self) -> dict:
        return {
            "c2_profile_name": self.Name,
            "parameters": self.Parameters
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2OPSECMessageResponse:
    """Status of running C2 Profile's opsec function

    Attributes:
        Success (bool): Did the OPSEC check succeed or fail
        Error (str): Error message if the OPSEC check failed
        Message (str): Informative message about the OPSEC check if there was no error

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 Message: str = "",
                 RestartInternalServer: bool = False,
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Message = Message
        self.RestartInternalServer = RestartInternalServer
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "message": self.Message
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2ConfigCheckMessage:
    """Payload's C2 Profile configuration for Configuration checking

    Attributes:
        Name (str): Name of the C2 Profile
        Parameters (dict): Dictionary of C2 Parameter name:value

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 c2_profile_name: str,
                 parameters: dict,
                 **kwargs):
        self.Name = c2_profile_name
        self.Parameters = parameters
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "c2_profile_name": self.Name,
            "parameters": self.Parameters
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2ConfigCheckMessageResponse:
    """Status of running C2 Profile's config_check function

    Attributes:
        Success (bool): Did the config check succeed or fail
        Error (str): Error message if the config check failed
        Message (str): Informative message about the config check if there was no error

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 Message: str = "",
                 RestartInternalServer: bool = False,
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Message = Message
        self.RestartInternalServer = RestartInternalServer
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "message": self.Message
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2GetIOCMessage:
    """Payload's C2 Profile configuration for getting IOCs

    Attributes:
        Name (str): Name of the C2 Profile
        Parameters (dict): Dictionary of C2 Parameter name:value

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 c2_profile_name: str,
                 parameters: dict,
                 **kwargs):
        self.Name = c2_profile_name
        self.Parameters = parameters
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "c2_profile_name": self.Name,
            "parameters": self.Parameters
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2GetIOCMessageResponseIOC:
    """An IOC to report back as part of a C2 Profile's get_ioc function

    Attributes:
        Type (str): The type of IOC (ex: Hash, URL)
        IOC (str): The actual IOC value
    """
    def __init__(self,
                 Type: str,
                 IOC: str):
        self.Type = Type
        self.IOC = IOC

    def to_json(self):
        return {
            "type": self.Type,
            "ioc": self.IOC
        }


class C2GetIOCMessageResponse:
    """Status of running C2 Profile's get_ioc function

    Attributes:
        Success (bool): Did the get ioc request succeed or fail
        Error (str): Error message if the get ioc request failed
        IOCs (List[C2GetIOCMessageResponseIOC]): List of the IOC values

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 IOCs: List[C2GetIOCMessageResponseIOC] = [],
                 RestartInternalServer: bool = False,
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.IOCs = IOCs
        self.RestartInternalServer = RestartInternalServer
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "iocs": self.IOCs
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2SampleMessageMessage:
    """Sample C2 message based on the Payload's configuration

    Attributes:
        Name (str): Name of the C2 Profile
        Parameters (dict): Dictionary of C2 Parameter name:value

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 c2_profile_name: str,
                 parameters: dict,
                 **kwargs):
        self.Name = c2_profile_name
        self.Parameters = parameters
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "c2_profile_name": self.Name,
            "parameters": self.Parameters
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2SampleMessageMessageResponse:
    """Sample C2 Message based on the payload's configuration

    Attributes:
        Success (bool): Did the sample message generation succeed or fail
        Error (str): Error message if the sample message generation failed
        Message (str): Sample message to represent that C2 traffic would look like for this payload

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 Message: str = "",
                 RestartInternalServer: bool = False,
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Message = Message
        self.RestartInternalServer = RestartInternalServer
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "message": self.Message
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2GetDebugOutputMessage:
    """Try to get debug output from subprocess running internal server code

    Attributes:
        Name (str): Name of the C2 Profile

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 c2_profile_name: str,
                 **kwargs):
        self.Name = c2_profile_name
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "c2_profile_name": self.Name,
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2GetDebugOutputMessageResponse:
    """Debug output from subprocesses running internal server

    Attributes:
        Success (bool): Were we able to get information from subprocess
        Error (str): Error message if the subprocess exited
        Message (str): Whatever output was generated from the subprocess
        InternalServerRunning (bool): Is the subprocess still running or not

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 Message: str = "",
                 InternalServerRunning: bool = False,
                 RestartInternalServer: bool = False,
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Message = Message
        self.InternalServerRunning = InternalServerRunning
        self.RestartInternalServer = RestartInternalServer
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "message": self.Message,
            "server_running": self.InternalServerRunning
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2GetRedirectorRulesMessage:
    """Payload's C2 Profile configuration for generating Apache ModRewrite rules

    Attributes:
        Name (str): Name of the C2 Profile
        Parameters (dict): Dictionary of C2 Parameter name:value

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 c2_profile_name: str,
                 parameters: dict,
                 **kwargs):
        self.Name = c2_profile_name
        self.Parameters = parameters
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "c2_profile_name": self.Name,
            "parameters": self.Parameters
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2GetRedirectorRulesMessageResponse:
    """Status of running C2 Profile's get redirector rules function

    Attributes:
        Success (bool): Did the redirector check succeed or fail
        Error (str): Error message if the redirector check failed
        Message (str): Apache ModRewrite rules

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 Message: str = "",
                 RestartInternalServer: bool = False,
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Message = Message
        self.RestartInternalServer = RestartInternalServer
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "message": self.Message
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2StartServerMessage:
    """Request to start a C2 Profile's internal server as a subprocess

    Attributes:
        Name (str): Name of the C2 Profile

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 c2_profile_name: str,
                 **kwargs):
        self.Name = c2_profile_name
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "c2_profile_name": self.Name,
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2StartServerMessageResponse:
    """Status of running C2 Profile's internal server as a subprocess

    Attributes:
        Success (bool): Did the start subprocess succeed or fail
        Error (str): Error message if the subprocess failed to run for more than 3 seconds
        Message (str): Output from the first 3 seconds of the subprocess running
        InternalServerRunning (bool): Is the internal server subprocess still running or not

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 Message: str = "",
                 InternalServerRunning: bool = False,
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Message = Message
        self.InternalServerRunning = InternalServerRunning
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "message": self.Message,
            "server_running": self.InternalServerRunning
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2StopServerMessage:
    """Request to stop the C2 Profile's internal server subprocess

    Attributes:
        Name (str): Name of the C2 Profile

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 c2_profile_name: str,
                 **kwargs):
        self.Name = c2_profile_name
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "c2_profile_name": self.Name,
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2StopServerMessageResponse:
    """Status of stopping the C2 Server's subprocess

    Attributes:
        Success (bool): Did the subprocesses successfully stop or not
        Error (str): Error message if the subprocess failed to stop
        Message (str): Any final output the subprocess was reporting
        InternalServerRunning (bool): Is the internal subprocess still running or not

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 Message: str = "",
                 InternalServerRunning: bool = False,
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Message = Message
        self.InternalServerRunning = InternalServerRunning
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "message": self.Message,
            "server_running": self.InternalServerRunning
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2OtherServiceRPCMessage:
    """Request to call an RPC function of another C2 Profile or Payload Type

    Attributes:
        ServiceName (str): Name of the C2 Profile or Payload Type
        ServiceRPCFunction (str): Name of the function to call
        ServiceRPCFunctionArguments (dict): Arguments to that function

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 ServiceName: str = None,
                 service_name: str = None,
                 ServiceRPCFunction: str = None,
                 service_function: str = None,
                 ServiceRPCFunctionArguments: dict = None,
                 service_arguments: dict = None,
                 **kwargs):
        self.ServiceName = ServiceName
        if self.ServiceName is None:
            self.ServiceName = service_name
        self.ServiceRPCFunction = ServiceRPCFunction
        if self.ServiceRPCFunction is None:
            self.ServiceRPCFunction = service_function
        self.ServiceRPCFunctionArguments = ServiceRPCFunctionArguments
        if self.ServiceRPCFunctionArguments is None:
            self.ServiceRPCFunctionArguments = service_arguments
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "service_name": self.ServiceName,
            "service_function": self.ServiceRPCFunction,
            "service_arguments": self.ServiceRPCFunctionArguments
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2OtherServiceRPCMessageResponse:
    """Result of running an RPC call from another service

    Attributes:
        Success (bool): Did the RPC succeed or fail
        Error (str): Error message if the RPC check failed
        Result (dict): Result from the RPC

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 success: bool = None,
                 error: str = None,
                 result: dict = None,
                 Success: bool = None,
                 Error: str = None,
                 Result: dict = None,
                 RestartInternalServer: bool = False,
                 **kwargs):
        self.Success = Success
        if self.Success is None:
            self.Success = success
        self.Error = Error
        if self.Error is None:
            self.Error = error
        self.Result = Result
        if self.Result is None:
            self.Result = result
        self.RestartInternalServer = RestartInternalServer
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "result": self.Result
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2HostFileMessage:
    """Request to host a file through the C2 profile's server

    Attributes:
        Name (str): Name of the C2 Profile
        FileUUID (str): UUID of the file to host
        HostURL (str): URL to host the file (ex: /bob)
        Remove (bool): If this path (or all paths with this uuid if path is empty) should be removed instead of added

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 c2_profile_name: str,
                 file_uuid: str,
                 host_url: str,
                 remove: bool,
                 **kwargs):
        self.Name = c2_profile_name
        self.FileUUID = file_uuid
        self.HostURL = host_url
        self.Remove = remove
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "c2_profile_name": self.Name,
            "file_uuid": self.FileUUID,
            "host_url": self.HostURL,
            "remove": self.Remove
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2HostFileMessageResponse:
    """Status of hosting a file

    Attributes:
        Success (bool): Did the file get hosted or not
        Error (str): Error message if the file failed be hosted

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(self,
                 Success: bool,
                 Error: str = "",
                 RestartInternalServer: bool = False,
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.RestartInternalServer = RestartInternalServer
        for k, v in kwargs.items():
            logger.error(f"unknown kwarg {k} {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class ParameterType(Enum):
    """Types of parameters available for C2 Profiles when building payloads

     Attributes:
         String:
             A string value
         ChooseOne:
             A list of choices for the user to select exactly one
         Array:
             The user can supply multiple values in an Array format
         Date:
             The user can select a Date in YYYY-MM-DD format
         Dictionary:
             The user can supply a dictionary of values
         Boolean:
             The user can toggle a switch for True/False
         TypedArray:
            The user can supply multiple values in an array format where each entry has a dropdown of choices for metadata
         File:
            The user can supply a file, but this will get passed along as a UUID representing the file
         Number:
            An number
     """
    String = "String"
    ChooseOne = "ChooseOne"
    ChooseOneCustom = "ChooseOneCustom"
    ChooseMultiple = "ChooseMultiple"
    Array = "Array"
    Date = "Date"
    Dictionary = "Dictionary"
    Boolean = "Boolean"
    TypedArray = "TypedArray"
    File = "File"
    FileMultiple = "FileMultiple"
    Number = "Number"


class DictionaryChoice:
    """A single dictionary choice/option for the UI when select C2 Profile Parameters

    Attributes:
        name (str): Name of the choice
        default_show (bool): Should this be pre-selected with a default value
        default_value (str): Default value to fill in

    """

    def __init__(self,
                 name: str,
                 default_value: str = "",
                 default_show: bool = True):
        self.name = name
        self.default_show = default_show
        self.default_value = default_value

    def to_json(self):
        return {
            "name": self.name,
            "default_value": self.default_value,
            "default_show": self.default_show
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2ProfileParameter:
    """C2Profile Parameter Definition for use when generating payloads

    Attributes:
        name (str): Name of the parameter for scripting and for when building payloads
        description (str): Informative description displayed when building a payload
        default_value (any): Default value to pre-populate
        randomize (bool): Should this value be randomized (requires format_string)
        format_string (str): A regex used for randomizing values if randomize is true
        parameter_type (ParameterType): The type of parameter this is
        required (bool): Is this parameter required to have a non-empty value or not
        verifier_regex (str): Regex used to verify that the user typed something appropriate
        choices (list[str]): Choices for ChooseOne parameter type
        dictionary_choices (list[DictionaryChoice]): Configuration options for the Dictionary parameter type
        crypto_type (bool): Indicate if this value should be used to generate a crypto key or not

    Functions:
        to_json(self): return dictionary form of class
    """

    def __init__(
            self,
            name: str,
            description: str,
            default_value: any = None,
            randomize: bool = False,
            format_string: str = "",
            parameter_type: ParameterType = ParameterType.String,
            required: bool = True,
            verifier_regex: str = "",
            choices: list[str] = None,
            dictionary_choices: list[DictionaryChoice] = None,
            crypto_type: bool = False,
    ):
        self.name = name
        self.description = description
        self.randomize = randomize
        self.format_string = format_string
        self.parameter_type = parameter_type
        self.required = required
        self.verifier_regex = verifier_regex
        self.choices = choices
        self.default_value = default_value
        self.crypto_type = crypto_type
        self.dictionary_choices = dictionary_choices

    def to_json(self):
        return {
            "name": self.name,
            "description": self.description,
            "default_value": self.default_value,
            "randomize": self.randomize,
            "format_string": self.format_string,
            "required": self.required,
            "parameter_type": self.parameter_type.value,
            "verifier_regex": self.verifier_regex,
            "crypto_type": self.crypto_type,
            "choices": self.choices,
            "dictionary_choices": [x.to_json() for x in
                                   self.dictionary_choices] if self.dictionary_choices is not None else None
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


class C2Profile:
    """C2 Profile to import and sync with Mythic

    Attributes:
        name (str): Name of the profile
        description (str): Description of the profile
        author (str): Author of the profile
        is_p2p (bool): Is the profile defining P2P communications or egress communications
        parameters (list[C2ProfileParameter]): Definition of the parameters for the profile used when building a payload
        server_folder_path (pathlib.Path): Path to the folder that contains `config.json`
        server_binary_path (pathlib.Path): Path to the binary that gets executed when "Start" is clicked in the UI
        custom_rpc_functions: Dictionary of RPC name to awaitable RPC function that other services can call

    Functions:
        opsec

        config_check

        redirect_rules

        get_ioc

        sample_message

        host_file

    """

    name: str = ""
    description: str = ""
    author: str = ""
    is_p2p: bool = False
    is_server_routed: bool = True
    parameters: [C2ProfileParameter] = []

    async def opsec(self, inputMsg: C2OPSECMessage) -> C2OPSECMessageResponse:
        """Check payload's C2 configuration for OPSEC issues

        :param inputMsg: Payload's C2 Profile configuration
        :return: C2OPSECMessageResponse detailing the results of the OPSEC check
        """
        response = C2OPSECMessageResponse(Success=True)
        response.Message = "Not Implemented, passing by default"
        #response.Message += f"\nInput: {json.dumps(inputMsg.to_json(), indent=4)}"
        return response

    async def config_check(self, inputMsg: C2ConfigCheckMessage) -> C2ConfigCheckMessageResponse:
        """Check a payload's C2 configuration to see if it matches the local configuration

        :param inputMsg: Payload's C2 Profile configuration
        :return: C2ConfigCheckMessageResponse detailing the results of the configuration check
        """
        response = C2ConfigCheckMessageResponse(Success=True)
        response.Message = "Not Implemented"
        response.Message += f"\nInput: {json.dumps(inputMsg.to_json(), indent=4)}"
        return response

    async def redirect_rules(self, inputMsg: C2GetRedirectorRulesMessage) -> C2GetRedirectorRulesMessageResponse:
        """Generate Apache ModRewrite rules given the Payload's C2 configuration

        :param inputMsg: Payload's C2 Profile configuration
        :return: C2GetRedirectorRulesMessageResponse detailing some Apache ModRewrite rules for the payload
        """
        response = C2GetRedirectorRulesMessageResponse(Success=True)
        response.Message = "#Not Implemented"
        return response

    async def get_ioc(self, inputMsg: C2GetIOCMessage) -> C2GetIOCMessageResponse:
        """Generate IOCs for the network traffic associated with the specified c2 configuration

        :param inputMsg: Payload's C2 Profile configuration
        :return: C2GetIOCMessageResponse detailing some IOCs
        """
        response = C2GetIOCMessageResponse(Success=True)
        response.IOCs = []
        return response

    async def sample_message(self, inputMsg: C2SampleMessageMessage) -> C2SampleMessageMessageResponse:
        """Generate a sample message for this c2 profile based on the configuration specified

        :param inputMsg: Payload's C2 Profile configuration
        :return: C2SampleMessageMessageResponse detailing a sample message
        """
        response = C2SampleMessageMessageResponse(Success=True)
        response.Message = "Not Implemented"
        #response.Message += f"\nInput: {json.dumps(inputMsg.to_json(), indent=4)}"
        return response

    async def host_file(self, inputMsg: C2HostFileMessage) -> C2HostFileMessageResponse:
        """Host a file through a c2 channel

        :param inputMsg: The file UUID to host and which URL to host it at
        :return: C2HostFileMessageResponse detailing success or failure to host the file
        """
        response = C2HostFileMessageResponse(Success=False)
        response.Message = "Not Implemented"
        #response.Message += f"\nInput: {json.dumps(inputMsg.to_json(), indent=4)}"
        return response

    async def on_container_start(self, message: ContainerOnStartMessage) -> ContainerOnStartMessageResponse:
        return ContainerOnStartMessageResponse(ContainerName=self.name)

    custom_rpc_functions: dict[
        str, Callable[[C2OtherServiceRPCMessage], Awaitable[C2OtherServiceRPCMessageResponse]]] = {}
    server_folder_path: pathlib.Path
    server_binary_path: pathlib.Path

    def to_json(self):
        if self.server_binary_path is None:
            logger.exception(
                "Must supply server_binary_path as pathlib.Path pointing to the binary to run when the server starts")
            sys.exit(1)
        elif not isinstance(self.server_binary_path, pathlib.Path):
            logger.exception("Wrong type for server_binary_path - should be pathlib.Path")
            sys.exit(1)
        elif self.server_folder_path is None:
            logger.exception(
                "Must supply server_folder_path as pathlib.Path pointing to folder where your c2 files exist")
            sys.exit(1)
        elif not isinstance(self.server_folder_path, pathlib.Path):
            logger.exception("Wrong type for server_folder_path - should be pathlib.Path")
            sys.exit(1)
        self.name = self.name.lower()
        return {
            "c2_profile": {
                "name": self.name,
                "description": self.description,
                "author": self.author,
                "is_p2p": self.is_p2p,
                "is_server_routed": self.is_server_routed,
            },
            "parameters": [x.to_json() for x in self.parameters]
        }

    def __str__(self):
        return json.dumps(self.to_json(), sort_keys=True, indent=2)


c2Profiles: dict[str, C2Profile] = {}
runningServers: dict[str, dict] = {}



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_CALLBACK_ADD_COMMAND = "mythic_rpc_callback_add_command"


class MythicRPCCallbackAddCommandMessage:
    def __init__(self,
                 Commands: list[str],
                 TaskID: int = None,
                 CallbackAgentUUID: str = None,
                 PayloadType: str = None,
                 CallbackIDs: list[int] = [],
                 **kwargs):
        self.TaskID = TaskID
        self.CallbackAgentUUID = CallbackAgentUUID
        self.Commands = Commands
        self.PayloadType = PayloadType
        self.CallbackIDs = CallbackIDs
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "commands": self.Commands,
            "callback_agent_id": self.CallbackAgentUUID,
            "payload_type": self.PayloadType,
            "callback_ids": self.CallbackIDs
        }


class MythicRPCCallbackAddCommandMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCCallbackAddCommand(
        msg: MythicRPCCallbackAddCommandMessage) -> MythicRPCCallbackAddCommandMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_CALLBACK_ADD_COMMAND,
                                                                            body=msg.to_json())
    return MythicRPCCallbackAddCommandMessageResponse(**response)



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CryptoKeysFormat(_message.Message):
    __slots__ = ["DecKey", "EncKey", "Value"]
    DECKEY_FIELD_NUMBER: _ClassVar[int]
    DecKey: bytes
    ENCKEY_FIELD_NUMBER: _ClassVar[int]
    EncKey: bytes
    VALUE_FIELD_NUMBER: _ClassVar[int]
    Value: str
    def __init__(self, EncKey: _Optional[bytes] = ..., DecKey: _Optional[bytes] = ..., Value: _Optional[str] = ...) -> None: ...

class TrCustomMessageToMythicC2FormatMessage(_message.Message):
    __slots__ = ["C2Name", "CryptoKeys", "Message", "MythicEncrypts", "TranslationContainerName", "UUID"]
    C2NAME_FIELD_NUMBER: _ClassVar[int]
    C2Name: str
    CRYPTOKEYS_FIELD_NUMBER: _ClassVar[int]
    CryptoKeys: _containers.RepeatedCompositeFieldContainer[CryptoKeysFormat]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    MYTHICENCRYPTS_FIELD_NUMBER: _ClassVar[int]
    Message: bytes
    MythicEncrypts: bool
    TRANSLATIONCONTAINERNAME_FIELD_NUMBER: _ClassVar[int]
    TranslationContainerName: str
    UUID: str
    UUID_FIELD_NUMBER: _ClassVar[int]
    def __init__(self, TranslationContainerName: _Optional[str] = ..., C2Name: _Optional[str] = ..., Message: _Optional[bytes] = ..., UUID: _Optional[str] = ..., MythicEncrypts: bool = ..., CryptoKeys: _Optional[_Iterable[_Union[CryptoKeysFormat, _Mapping]]] = ...) -> None: ...

class TrCustomMessageToMythicC2FormatMessageResponse(_message.Message):
    __slots__ = ["Error", "Message", "Success", "TranslationContainerName"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    Error: str
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    Message: bytes
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    Success: bool
    TRANSLATIONCONTAINERNAME_FIELD_NUMBER: _ClassVar[int]
    TranslationContainerName: str
    def __init__(self, Success: bool = ..., Error: _Optional[str] = ..., Message: _Optional[bytes] = ..., TranslationContainerName: _Optional[str] = ...) -> None: ...

class TrGenerateEncryptionKeysMessage(_message.Message):
    __slots__ = ["C2Name", "CryptoParamName", "CryptoParamValue", "TranslationContainerName"]
    C2NAME_FIELD_NUMBER: _ClassVar[int]
    C2Name: str
    CRYPTOPARAMNAME_FIELD_NUMBER: _ClassVar[int]
    CRYPTOPARAMVALUE_FIELD_NUMBER: _ClassVar[int]
    CryptoParamName: str
    CryptoParamValue: str
    TRANSLATIONCONTAINERNAME_FIELD_NUMBER: _ClassVar[int]
    TranslationContainerName: str
    def __init__(self, TranslationContainerName: _Optional[str] = ..., C2Name: _Optional[str] = ..., CryptoParamValue: _Optional[str] = ..., CryptoParamName: _Optional[str] = ...) -> None: ...

class TrGenerateEncryptionKeysMessageResponse(_message.Message):
    __slots__ = ["DecryptionKey", "EncryptionKey", "Error", "Success", "TranslationContainerName"]
    DECRYPTIONKEY_FIELD_NUMBER: _ClassVar[int]
    DecryptionKey: bytes
    ENCRYPTIONKEY_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    EncryptionKey: bytes
    Error: str
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    Success: bool
    TRANSLATIONCONTAINERNAME_FIELD_NUMBER: _ClassVar[int]
    TranslationContainerName: str
    def __init__(self, Success: bool = ..., Error: _Optional[str] = ..., EncryptionKey: _Optional[bytes] = ..., DecryptionKey: _Optional[bytes] = ..., TranslationContainerName: _Optional[str] = ...) -> None: ...

class TrMythicC2ToCustomMessageFormatMessage(_message.Message):
    __slots__ = ["C2Name", "CryptoKeys", "Message", "MythicEncrypts", "TranslationContainerName", "UUID"]
    C2NAME_FIELD_NUMBER: _ClassVar[int]
    C2Name: str
    CRYPTOKEYS_FIELD_NUMBER: _ClassVar[int]
    CryptoKeys: _containers.RepeatedCompositeFieldContainer[CryptoKeysFormat]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    MYTHICENCRYPTS_FIELD_NUMBER: _ClassVar[int]
    Message: bytes
    MythicEncrypts: bool
    TRANSLATIONCONTAINERNAME_FIELD_NUMBER: _ClassVar[int]
    TranslationContainerName: str
    UUID: str
    UUID_FIELD_NUMBER: _ClassVar[int]
    def __init__(self, TranslationContainerName: _Optional[str] = ..., C2Name: _Optional[str] = ..., Message: _Optional[bytes] = ..., UUID: _Optional[str] = ..., MythicEncrypts: bool = ..., CryptoKeys: _Optional[_Iterable[_Union[CryptoKeysFormat, _Mapping]]] = ...) -> None: ...

class TrMythicC2ToCustomMessageFormatMessageResponse(_message.Message):
    __slots__ = ["Error", "Message", "Success", "TranslationContainerName"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    Error: str
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    Message: bytes
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    Success: bool
    TRANSLATIONCONTAINERNAME_FIELD_NUMBER: _ClassVar[int]
    TranslationContainerName: str
    def __init__(self, Success: bool = ..., Error: _Optional[str] = ..., Message: _Optional[bytes] = ..., TranslationContainerName: _Optional[str] = ...) -> None: ...



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_C2_UPDATE_STATUS = "mythic_rpc_c2_update_status"


class MythicRPCC2UpdateStatusMessage:
    """
    Update a C2 profile's status on the Mythic server about if the internal server is running or not
    """

    def __init__(self,
                 C2Profile: str,
                 InternalServerRunning: bool,
                 Error: str = None,
                 **kwargs):
        self.C2Profile = C2Profile
        self.InternalServerRunning = InternalServerRunning
        self.Error = Error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "c2_profile": self.C2Profile,
            "server_running": self.InternalServerRunning,
            "error": self.Error,
        }


class MythicRPCC2UpdateStatusMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCC2UpdateStatus(
        msg: MythicRPCC2UpdateStatusMessage) -> MythicRPCC2UpdateStatusMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_C2_UPDATE_STATUS,
                                                                            body=msg.to_json())
    return MythicRPCC2UpdateStatusMessageResponse(**response)
